﻿<?php
// libs/a/a.charset.php
// (c) Yuri Popoff, Nov 2003, popoff.donetsk.ua
// A set of functions to process charsets

define('A_CHARSET_MARK_BAD',128);
define('A_CHARSET_MARK_GOOD',-1);
define('A_CHARSET_MARK_PAIR',32);

function _a_charset_ru_bad($s)
{ //count "bad" symbols in russian, in windows-1251
  $r=0;
  for($i=0;$i<strlen($s);$i++)
  {
    if(ord($s[$i])==0x80)
      $r++;
  }
  return $r;
}

function _a_charset_ru_cp1251($s)
{
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=0xc0&&$c<=0xdf)
      $s[$i]=chr($c+0x20);
    elseif($c==0xa8||$c==0xb8)
      $s[$i]='е';
    elseif(!($c>=0xe0&&$c<=0xff))
      $s[$i]=chr(0x80); //Any invalid character for CP1251
  }
  return $s;
}

function _a_charset_ru_cp866($s)
{
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=0x80&&$c<=0x9f)
      $s[$i]=chr($c+0x60);
    elseif($c>=0xa0&&$c<=0xaf)
      $s[$i]=chr($c+0x40);
    elseif($c>=0xe0&&$c<=0xef)
      $s[$i]=chr($c+0x10);
    elseif($c==0xf0||$c==0xf1)
      $s[$i]='е';
    else
      $s[$i]=chr(0x80); // An indicator for an invalid character
  }
  return $s;
}

function _a_charset_ru_good($s)
{ //count "good" symbols in russian, in windows-1251
  $r=0;
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=0xe0)
      $r++;
  }
  return $r;
}

function _a_charset_ru_koi($s)
{
  $kw = array(
    //00    01    02    03    04    05    06    07    08    09    0a     0b    0c     0d    0e    0f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x80 - 0x8f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x90 - 0x9f
    0x80, 0x80, 0x80,  229, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xa0 - 0xaf
    0x80, 0x80, 0x80,  229, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xb0 - 0xbf
     254,  224,  225,  246,  228,  229,  244,  227,  245,  232,  233,   234,  235,   236,  237,  238, //0xc0 - 0xcf
     239,  255,  240,  241,  242,  243,  230,  226,  252,  251,  231,   248,  253,   249,  247,  250, //0xd0 - 0xdf
     254,  224,  225,  246,  228,  229,  244,  227,  245,  232,  233,   234,  235,   236,  237,  238, //0xe0 - 0xef
     239,  255,  240,  241,  242,  243,  230,  226,  252,  251,  231,   248,  253,   249,  247,  250  //0xf0 - 0xff
     );
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=128)
      $s[$i]=chr($kw[$c-128]);
    else
      $s[$i]=chr(128);
  }
  return $s;
}

function _a_charset_ru_pair($s)
{ //count "bad" pairs of chars for a string in russian, in windows-1251
  $a=array(
    '0' => 'ъыь',
    '1' => 'йфэ',
    '2' => 'йэ',
    '3' => 'жйпфхцщъыьэя',
    '4' => 'й',
    '5' => 'ъыь',
    '6' => 'йтхшщъыэя',
    '7' => 'йфхщ',
    '8' => 'ъыь',
    '9' => 'жийуъыьэю',
    '10' => 'бйпфщъыьэя',
    '11' => 'йръэ',
    '12' => 'джйъ',
    '13' => 'йм',
    '14' => 'ъыь',
    '15' => 'бвгджзйъэ',
    '16' => 'йъ',
    '17' => 'й',
    '18' => 'й',
    '19' => 'ъыь',
    '20' => 'бвджзйкпхцшщъэ',
    '21' => 'жйфхщыьюя',
    '22' => 'бгджзйлрстфхцчшщъьэюя',
    '23' => 'бгджзйпфхцчщъыэюя',
    '24' => 'бгджзйфхшщъыэя',
    '25' => 'бвгджзйклмпстфхцчшщъыэюя',
    '26' => 'абвгджзийклмнопрстуфхцчшщъыьэ',
    '27' => 'аофъыьэю',
    '28' => 'айлрухъыьэ',
    '29' => 'абежиоуцчщъыьэю',
    '30' => 'уъыьэ',
    '31' => 'аофъыьэ'
    );
  $b=array(
    0 => 'ааабавадаеажазаиайаоауахачашащаэаюаябгбмбпбтбцбчбъбювжвфвъгзгкгтгчгюдщдэеаебеееиейелеоефещеэеюеяжбжвжзжлжпжфжцжчжюзпзтзшиаиеижииийиоиуифицищиэиюиякдкзлзлнлшлщмхмэоаогоеожойомоооуофоцошощоэоюояпмпхпцпщрэсгсдсжсзсрсэтбтгтдтжтзтптштщтъуауеуиуйуоуууфуюуяфмфюфяхбхгхдхзхпхцхъхэчвчрчшшршчщрщьэвэгэзэйэкэмэнэпэфэхэшэяюаюбювюгюдюеюжюзюиюйюмюнюоюпюрюфюцюююяягяжяияйямяпяуяхяш',
    1 => 'ааафащаэбббвбгбдбжбзбкблбмбнбпбтбубхбцбчбшбщбъбыбьбюбявбвввгвдвжвзвквлвмвнвпврвсвтвувфвхвцвчвшвщвъвьвюгбгвгггдгегзгигкгмгнгсгтгугчгшгюдбдвдгдддждздкдлдмдндпдрдсдтдфдхдцдчдшдщдъдыдьдэдюепеуехеэжбжвжгжджжжзжкжлжмжнжпжржсжфжцжчжьжюзбзгздзезжзззкзлзмзнзпзрзсзтзузцзчзшзъзьзэзюзяиииэиюквкгкдкекжкзкккмкнкскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщмбмвмгмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэпапепипкплпмпнпппсптпупфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпррртрфрхрцрчршрщрэсбсгсдсжсзснспсрсссфсхсшсщсъсьсэсютбтвтгтдтжтзтктлтмтнтптстттфтхтцтчтштщтътьтэуауоупуухбхгхдхехзхкхлхмхнхпхрхсхтхухцхчхшхъхэцацвцкцмцнцпчвчкчлчмчнчрчсчтчучшчьшвшкшмшншошпшршсшушцшчшьшющрщущьъюыбыеыжыиымыпырыуыхыцыяьбьвьгьдьжьзькьмьньоьпьсьтьфьцьчьшьщюаюбюгюеюжюзюиюйюмюнюоюпюфюхюцючюшющюююяябявягядяеяияйямяпярясяуяхяцяшяюяя',
    2 => 'аоаэбббвбгбдбжбзбкбмбнбпбсбтбхбцбчбшбщбъбыбьбюбявбвввгвдвжвзвквлвмвнвпвсвтвувфвхвцвчвшвщвъвьвюгагбгвгггдгзгкгмгнгсгтгчгшгюдбдгдддждздкдлдмдндпдсдтдфдхдцдчдшдщдъдьдэдюдяеаеуефеэжбжвжгжджежжжзжкжлжмжнжожпжржсжужфжцжчжьжюзезжзззкзпзсзтзузцзчзшзьзэзюиэиюквкгкдкекжкзкккмкскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлулфлхлцлчлшлщлымбмвмгмзмкмлмммнмрмсмтмфмхмцмчмшмщмымьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэпкпмпнпппсптпфпхпцпчпшпщпыпьпюрбрвргрдржрзркрлрмрпррртрфрхрцрчршрьрэсбсвсгсдсжсзснсрсссфсцсчсшсщсъсьсэтбтвтгтдтжтзтктлтмтнтптстттфтхтцтчтштщтътьтэувуйуфуэфафгфифлфмфнфсфтфуфффчфыфьфюфяхбхвхгхдхехзхихкхлхмхнхпхрхтхухцхчхшхъхэцвцицкцмцнцпчвчкчлчмчнчрчсчтчшчьшвшкшлшмшншпшршсштшушцшчшьшющащещнщощрщущьъюьбьвьгьдьжьзькьмьньоьпьсьтьфьцьчьшьщюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюсютюфюхюцючюшющюююяябявягяеяияйяпяряуяцяюяя',
    3 => 'ахаэбббвбгбдбжбзбкблбмбнбобпбрбсбтбубхбцбчбшбщбъбыбьбюбявбвввгвдвжвзвквлвмвнвпврвсвтвувфвхвцвчвшвщвъвювягагбгвгггдгзгигкгмгнгсгтгугчгшгюдбдвдгдддждздкдлдмдндпдсдудфдхдцдчдшдщдъдьдэдюдяеаежеиеуехецечещеэеюзбзвзгздзезжзззизкзлзмзнзозпзрзсзтзузцзчзшзъзызьзэзюзяижиииуифищиэиюквкгкдкжкзккклкмкнкркскткукхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщмбмвмгмзмкмлмммнмпмрмсмтмумфмхмцмчмшмщмымьмэмюмянбнвнгнднжнзнкнлнннпнрнснтнфнхнчншнщнъныньнэоюрбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрьрэсасвсгсдсесжсзсислсмснсоспсрсссусфсхсцсчсщсъсысьсэсютбтвтгтдтжтзтктлтмтнтптртттутфтхтцтчтштщтътытьтэуйуоуууфухуэуячвчкчлчмчнчочрчсчтчучшчьшашвшкшлшмшншошпшршсшушцшчшьшююаюбювюгюдюеюжюзюиюкюлюмюнюоюпюсютюфюхюцючюшющюююя',
    4 => 'аааэбббвбгбдбжбзбкблбмбнбпбсбтбубхбцбчбшбщбъбыбьбюбявбвввгвдвжвмвнвпврвсвтвфвхвцвчвшвщвъвьвюгбгвгггдгзгкгмгсгтгчгшгюдбдгдддждздкдлдмдндпдсдтдфдхдцдчдшдщдъдыдьдэдюдяеуещжбжвжгжджжжзжкжнжпжржфжцжчжьжюзбзгздзжзззкзлзмзпзрзсзтзцзчзшзъзэзяиюквкгкдкжкзкккмкнкскткхкцкчкшкюлблвлглдлжлзлклллмлнлплтлфлхлцлчлшлщльмбмвмгмзмкмлмммпмрмсмтмфмхмцмчмшмщмьмэмюнбнвнгнднжнзнлнннпнрнснтнфнхнцнчншнщнъньнэоаоюпкпмпнпппсптпфпхпцпчпшпщпыпьпюрбрвргрдржрзркрлрмрнрпррртрфрхрцрчршрщрьрэсбсгсдсжсзсрсфсхсцсшсщсъсьсэсютбтгтдтжтзтктлтмтнтптттфтхтцтчтштщтътьтэтюуффгфлфмфнфрфсфтфуфффчфыфьфюфяхбхгхдхехзхкхмхнхпхсхтхухцхчхшхъхэцицкцмцнцпцучвчкчлчмчнчрчсчтчшчьшвшмшршсшцшчшьшющащнщощрщущьыеызыиыпыуыцыяьвьгьдьжьзьиьньоьпьтьфьцьчьшьщэвэдэзэйэкэлэмэнэпэрэсэфэхэшэяюаювюдюеюмюоюпютюфюхюцюююяябявяеязяияляпяряуяхяцяюяя',
    5 => 'аааеажаиайаоасауафахачашащаэаюаябббвбгбдбжбзбмбпбхбъвбвввжвъгвгзгюдхдцдщдэеаееежеиейеуефещеэеюжюзпиаибиеижииикипириуифицишиэиюияйвйякгкжкмкхлблдлжлзлмлплхлцлшлщмвмхмэмюнжнъоеойооошоюояпмпфпхсзсэсютжтзтфтщтътэуауиуйуоуууфуцушущуэфгфмфыфьфюфяцмцычвчмчрчсчшшпшршцшчшюэвэгэдэзэйэлэнэпэрэхэшэяюаюбювюеюжюзюиюйюкюлюмюоюпюрюфюхюцюшюююяябяеяжяияйякямяпяуяхяцячяшяюяя',
    6 => 'ааагаиаоапауафацашаэаябббвбгбдбебжбзбкбмбнбобпбсбтбубхбцбчбшбщбъбыбьбявбвввгвдвжвзвквлвмвнвовпврвсвтвфвхвцвчвшвщвъвывьвювягбгвгггдгегзгкгмгнгргсгтгчгшгюдбдвдгдддждздкдмдндпдрдсдтдфдхдцдчдшдщдъдэдюдяежефехеэжбжвжгжджжжзжкжлжмжнжпжржсжфжцжчжьжюзбзгздзезжзззизкзлзмзнзпзрзсзтзцзчзшзъзызьзэзюзяиаифиэкгкдкжкзкккмкнкскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлулфлхлцлчлшлщлыльлюлямбмвмгмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмьмэмюмянбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюоаодоеожоиооофоцошощоэояпепипкпмпнпппсптпупфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрорпрррсртрфрхрцрчршрщрырьрэрюрясасбсвсгсдсжсзслсмснспсрсссусфсхсцсчсшсщсъсысьсэсюсяубувугудумуоуууфуцушуэфгфефифлфмфнфофсфтфуфффчфыфьфюфяцацвцицкцмцнцоцпцуцычачвчкчлчмчнчочрчсчтчучшчььбьвьгьдьжьзькьмьньоьпьтьфьцьчьшьщюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюсютюфюхюцючюшющюююя',
    7 => 'аэбббвбгбдбжбзбкбмбнбпбсбтбхбцбчбшбщбъбьбювбвввгвдвжвзвмвпвсвтвфвхвцвчвшвщвъвюгбгвгггдгзгкгмгсгтгчгюдбдгдддждздлдмдпдсдтдфдхдшдщдъдэдюеаегежезесеуечещеэжбжвжджзжкжлжмжнжпжржсжфжцжчжьжюзбзгздзжзззизкзмзпзсзтзцзчзшзъзызьзэзюихиюквкгкдкекжкзккклкмкнкркскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщльмбмвмгмзмкмлмммпмрмсмтмфмхмцмчмшмщмьмэмюнбнвнгнднжнзнкнлнннпнснтнфнхнцнчншнщнънэоеоюпапипкплпмпнпппсптпупфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюсасбсвсгсдсесжсзсислсмснсоспсрсссусфсхсцсчсшсщсъсысьсэсютатбтвтгтдтжтзтктлтмтнтотптртстттутфтхтцтчтштщтътытэтютяуоуууцуэуяцацвцицкцмцнцпцуцычвчкчлчмчнчочрчсчтчучшчьшашвшешкшлшмшншошпшршсштшушцшчшьшюыеыжыиыпыуыхыцышыяьвьгьдьжьзьиьньоьпьсьтьфьцьчьшьщьюэвэгэдэзэйэнэпэрэсэфэхэшэяюаювюгюеюжюиюнюоюпюрюсютюфюхюцючюшющюююяягядяжязялямяпярясяуяхяцяшяя',
    8 => 'аааеажаиайауащаэаюаябгбжбзбмбпбтбхбцбъбявввгвжвпвтвфвъвюгбгкгтгчгюдгдмдпдфдхдшдщдъдэеаебееежеиекеуефечещеэеюеяжбжвжзжлжпжржфжцжчжьжюзчзэиаибивигидиеижииийикиоипиуифихицичишищиэиюияйайбйвйгйдйейзйлйпйрйхйчйшйякгкдкжкзлвлжлтлчлшлщмгмзмтмхмшмэмюнроеойощоюояпмпфпхпщрбрфрэсжсзсрсщсъсэтдтжтхтштщтъуаувуеужузуиуйулуоуууфухуцушущуэуюуяфгфыфьфюфяхгхдхзхкхцхчхъхэцнцпчвчрчшшршцшчшющьэвэгэзэйэмэпэрэсэхэшэяюаюбювюгюдюеюжюзюиюйюкюоюпюрюсюфюхюцюшюююяябягяеяияйяляпярясяуяцяшящяя',
    9 => 'ааабавагадаеажаиайакаланаоапарасатауафахацачашащаэаюаябббвбгбдбебжбзбкблбмбпбрбсбтбхбцбчбшбщбъбьбюбявбвввгвдвжвзвивквлвмвнвпврвсвтвувфвхвцвчвшвщвъвывьвювягбгвгггдгзгигкглгмгнгогргтгчгшгюдбдгдддздкдлдмдндпдрдсдтдфдхдцдчдшдщдъдыдьдэдюеаебевегедееежезеиейекелеоепесетеуефехецечешещеэеюеязбзвзгздзжзззкзлзмзнзозпзрзсзтзузцзчзшзъзызьзэзюзяквкгкдкжкзкккмкркскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлулфлхлцлчлшлщлюлямбмвмгмзмкмммнмпмрмтмфмхмцмчмщмьмэмюнбнгнднжнзнкнлнннпнрнунфнхнчнщнъньнэнюоаобоеожоиойооопоуофоцочощоэоюояпапкпмпппрпспупхпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрырьрэрюрясасвсгсдсжсзслспсрсусхсцсчсщсъсысьсэсютбтвтгтдтжтзтктптттфтхтцтчтштщтътытьтэтютяфгфлфмфнфрфсфтфффчфыфьфюфяхбхвхдхехзхкхлхнхпхрхтхухчхшхъхэцвцкцмцнцпчвчкчлчмчнчрчсчтчучшчьшвшкшлшмшншошпшршсштшушцшчшьшющащещнщощрщущьябягядяеяжязяияйякямяняпярясятяуяхяцячяшящяюяя',
    10 => 'ащвбвввгвдвжвзвквлвмвнвпврвсвтвфвхвцвчвшвщвъвывьвюгбгвгггдгегзгигкглгмгнгсгтгугчгшгюдбдвдгдддедждздидкдлдмдндпдрдсдтдудфдхдцдчдшдщдъдыдьдэдюдяевееежеоеуехещеэеюеяжажбжвжгжджжжзжижкжлжмжнжожпжржсжужфжцжчжьжюзбзвзгздзжзззкзлзмзнзпзрзсзтзузцзчзшзъзызьзэзюзяигиииуицищиэиюквкгкдкжкзкккмкнкткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщмбмвмгмзмимкмлмммнмпмрмсмтмфмхмцмчмшмщмымьмэмюмянанбнвнгнжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюрбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрьрэсбсжсзсрсхсчсшсщсъсьсэтбтвтгтдтжтзтктлтмтптттфтхтцтчтштщтътьтэужуэуяхбхвхгхдхзхкхлхнхохпхрхсхтхухцхчхшхъхэцацвцкцмцнцоцпцуцычвчкчлчмчнчочрчсчтчучшчьшвшкшлшмшншошпшршушцшчшьшююаюбюгюдюеюжюзюиюйюкюнюоюпютюфюхюцючюшющюююя',
    11 => 'аэбббвбгбдбжбзбкбмбпбсбтбхбшбщбъбьбювбвввгвдвжвзвмвпврвувфвхвцвчвшвщвъвывьвювягбгвгггдгзгкгмгнгсгтгчгшгюдбдгдддждздкдлдмдпдрдсдтдфдхдцдчдшдщдъдьдэжбжвжгжджжжзжкжлжмжпжржфжцжчжюзбзвзгздзжзззлзмзпзрзцзчзъзызьзэзюиюкгкдкжкзкккмкскткхкцкчкюлблвлглдлжлзлклллмлнлплтлфлхлцлчлшлщмбмвмгмзмкмммнмпмрмтмфмхмцмчмшмщмьмэмюмянбнвнгнднжнзнкнлнннпнрнснтнфнхнчншнщнъньнэнюпкпмпнпппсптпфпхпцпчпшпщпыпюсгсдсжсзснспсфсхсцсчсшсщсъсьсэсютбтвтгтдтжтзтмтптстттфтхтцтчтштщтътэтюфгфифлфмфнфрфсфффчфыфьфюфяхбхгхдхехзхкхмхнхпхрхсхтхухцхчхшхъхэцвцицкцмцнцоцпцуцычвчлчмчрчсчтчшшвшишлшмшншошпшршсшушцшчшьшющнщрщущьызыиылырыуыцыяюа',
    12 => 'аабббвбгбжбзбкбмбпбсбтбхбцбчбшбщбъбыбьбювбвввгвдвевжвквлвмвнвпвсвтвувфвхвцвчвшвщвъвывьвювягбгвгггдгзгигкгмгргсгтгугчгшгюепеуеэзбзвзгзжзззкзлзмзнзпзрзсзтзузцзчзшзъзызьзэзюзяиииюквкгкдкжкзккклкмкскткхкцкчкшкюлблвлглдлжлзлклллмлнлолплслтлулфлхлцлчлшлщлымбмвмгмзмлмммпмрмсмтмфмхмцмшмщмымьмэмянбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэпкпмпнпппхпчпшпщпыпюпярбрвргрдржрзркрлрмрнрорпррртрфрхрцрчршрщрьрэрюрясбсвсгсдсжсзслсмспсрсфсхсцсчсшсщсъсысэсютбтвтгтдтжтзтктлтмтнтптртттутфтхтцтчтштщтътытьтэтютяубувуйуоупуууяфгфмфнфрфсфтфуфффчфыфьфюфяхбхвхгхдхзхкхлхмхнхпхрхсхтхцхчхшхъхэцацвцицкцмцнцпцуцычвчкчлчмчнчочрчсчтчшчьшвшкшлшмшншпшршсшушцшчшьшющащнщощрщущьыбыдыиыпыуыщыяьбьвьгьжьзьиькьмьньоьпьтьфьцьчьшьщэвэгэдэзэйэкэлэмэпэсэфэхэшэяюаюбювюгюдюеюжюиюйюкюмюпюсютюфюхюцючюшющюююяябяжязяйяхяюяя',
    13 => 'бббвбгбдбжбзбкбмбнбпбсбтбхбцбчбшбщбъбыбьбюбявбвввгвдвжвзвквлвмвнвпвсвтвфвхвцвчвшвщвъвывьвювягбгггдгзгнгчгюдддпдтдчдщдъдьдэдяжбжвжгжджжжзжкжлжмжнжпжржсжфжцжчжьжюзбзвзгздзжзззкзлзмзнзпзрзсзтзцзчзшзъзызьзэзюиэкгкдкжкзкккхкчкшкюлблвлглдлелжлзлклллмлнлплслтлулфлхлцлчлшлщлыльлюнбнвнгнднжнзнкнлнннпнрнтнфнхнцнчншнщнъньнэпкпмпнппптпупфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпрррсртрурфрхрцрчршрщрырьрэрюрясдсжсзсрсхсчсшсщсэсятбтдтжтзтптттфтхтцтчтъуйуоупуцфгфмфнфтфффчфыфьхбхвхгхдхзхкхлхмхнхпхсхтхцхчхшхъхэцвцнчвчкчмчнчрчтчшчьшкшмшошпшршсшушцшчшьшющащещнщощрщущьъяыбыгыдыжызыиыпысыуыцыщыяьбьвьзьмьньпьфьцьщэвэгэдэзэйэкэлэмэтэфэхэшэяюбювюеюжюзюиюйюкюлюмюоюпюфюцюяябядяияпяряу',
    14 => 'ааабаеажаиайаоацашащаюаябпбцвжвфвъвюгггзгкгчгюеаеееиеуефещеэеяжвжлжпжфжчжюзсзшзъзэиаибижииийипиуифиэиюияйайгйзйрйфйхйшкжкхкчкюмюнэоеоиойоуошоэоюояпмпфпхпюсщтюуауеуиуйуоуууфуцущуэфффчфюфяхгхдхзхкхпхцхъхэцнцпцуцычрчсчшшршцшчэгэйэсэхэшэяюаюбювюгюжюиюйюкюлюмюнюоюпюфюхюцючюшягяияйямяпяуяхяцяя',
    15 => 'аааоаэеаебеуефеэибиэквкгкдкекжкзккклкмкнкркскткхкцкчкшкюлблвлглдлжлзлклллнлплслтлфлхлцлчлшлщмбмвмгмемзмимкмлмммнмомпмрмсмтмумфмхмцмчмшмщмымьмэмюмянбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюпмпнпппсптпфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпррртрфрхрцрчршрщрэсбсвсгсдсжсзслсмснспсрсфсхсцсчсшсщсъсьсэтбтгтдтжтзтктлтмтптттфтхтцтчтштщтътьтэтюувуеужуиуйуоуууяфгфлфмфнфофсфтфуфффчфыфьфюфяхахбхвхгхдхзхихкхлхмхохпхрхсхтхухцхчхшхъхэцацвцкцмцнцпчвчкчлчмчнчрчсчтчшчьшвшкшлшмшншпшршсштшушцшчшьшющнщощрщьыбыдыеызыиыкымыпысыуыцыяьбьвьгьдьжьзькьмьньоьпьсьтьфьцьчьшьщюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюсюфюхюцючющюююяябягяеяжязяияйякяпяуяцяшяюяя',
    16 => 'бббвбгбдбжбзбмбпбтбхбчбшбщбъвбвввгвдвжвзвквмвпврвтвфвхвцвчвшвщвъгбгдгзгмгчгюдбдгдддздкдмдпдрдтдфдхдшдъдэжбжвжгжжжзжлжмжпжржфжчжюзбзгздзжзззмзпзрзтзцзчзъзэзюкдкжкзкркхкцкчкюлблвлглдлжлзлклллмлнлплтлфлхлцлчлшлщмбмвмгмзмммпмрмтмхмцмэмюнвнгнднжнзнлнннрнтнфнхнчнщнънэпмпппсптпфпхпцпчпшпщпьпюрбрвргрдржрзркрлрмрнрпррртрфрхрцрчршрщрырьрэрюрясбсгсдсжсзслсмсрсссхсцсчсшсщсъсьсэсютгтхтщтътэууфгфмфрфффчфыхццмцнцпчвчнчрчсчтчшшвшпшсшцшчшьшющнщрщущьыиырыуыяьвьдьжьзьньоьпьфьцьшьщэвэгэдэзэйэмэнэпэрэсэфэхэшэяюаюбювюдюеюйюоюрюцюяяияряу',
    17 => 'ащбббвбгбдбжбзбкбмбнбпбсбтбхбцбчбшбщбъбьбявбвввгвдвжвзвквлвмвнвпврвсвтвувфвхвцвшвщвъвьвюгбгвгггдгзгкгмгсгтгчгшгюдбдгдддждздкдлдмдндпдсдтдфдхдцдчдшдщдъдыдьдэдюдяежеэжбжвжгжджжжзжкжлжмжнжожпжржсжфжцжчжьжюзбзвзгздзезжзззизкзлзмзнзозпзрзсзтзузцзчзшзъзьзэзюзяиукгкдкжкзкккмкткхкцкчкшкюлблвлглдлжлзлклллмлнлплтлфлхлцлчлшлщмбмвмгмзмкмлмммнмпмсмтмфмхмцмчмшмщмэмюнбнвнднжнзнкнлнннпнрнснтнфнхнцнчншнщнънэоэпкпмппптпфпхпцпчпшпщпюрбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюсгсдсжсзсфсхсцсшсщсъсьсэтдтжтзтхтштщтъуоуууцфгфлфмфнфффчфьфюфяхбхгхдхзхкхмхпхсхтхцхчхшхъхэцкцмцнцпчвчкчмчнчрчсчтчшчьшкшмшпшршсшцшчшьшющащищнщощрщущьъюыбыдыеыжыиыуыцышыяьвьгьдьжьзьиьньпьсьтьфьцьчьшьщьюэвэгэдэзэйэлэнэпэтэфэхэшэяюаюбювюеюйюнюоюпютюфюцючющюююяябявяеяияйяляпяряуяхяшяюяя',
    18 => 'аабббвбгбдбжбзбкбмбнбпбсбтбхбцбчбшбщбъбьбявбвввгвдвжвквмвнвпвсвтвфвхвцвчвшвщвъвюгбгвгггдгзгкгмгнгсгтгчгшгюдбдвдгдддждздкдлдмдндпдсдтдфдхдцдчдшдщдъдьдэдюдяеэжбжвжгжджжжзжкжлжмжнжожпжржсжфжцжчжьжюзбзгздзезжзззизкзлзмзнзпзрзсзтзузцзчзшзъзьзэзюзяиюквкгкдкжкзкккмкскткхкцкчкшкюлблвлглдлжлзлклллмлнлплтлфлхлцлчлшлщмбмвмгмзмкмлмммнмпмрмтмфмхмцмчмшмьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэпкпмпнппптпфпхпцпчпшпщпыпьпюржрзркрлртрхрцрчршрэсгсжсзсфсхсцсшсщсъсьсэтбтвтгтдтжтзтктлтптттфтхтцтчтштщтътьтэтюфгфлфмфнфрфсфтфффчфьфюфяхбхгхдхехзхкхмхнхпхрхсхтхухцхчхшхъхэцкцмцнцпцучвчкчмчнчочрчсчшчьшкшмшншошпшршсшцшчшьшющнщощрщущьызыиыпыуыцыяьгьжьзьньоьпьцьчьшьщэвэгэдэзэйэлэнэпэрэфэхэшэяювюеюиюйюоюпюсюцющюююяядяеяияляуяюяя',
    19 => 'ааабагадаеажаиаоауафачащаэаюаябзбхбшбювбвввгвдвжвмвпвтвфвхвцвчвъвюгбгдгзгкгмгтгчгшгюдбдгдпдфдхдцдчдщдъеаебегееежеиейенеуефещеэеюеяжвжзжпжржфжцжюзжзззмзпзтзъзэзюиаибигиеижииийиоипиуифихичишищиэиюияйайзйлйойпйрйфйхйцйшкгкдкжкзкхкюлблвлглжлзлмлнлплфлхлцлчлшлщмзмтмхмшмэмюнбнлнпнрнфнънэоаогоеожоиойомоноооуохоцочошощоэоюояпмпфпхпшрэсдсжсзсхсшсщсъсэсютдтжтзтмтптфтхтцтштщтътэуаубувудуеужузуиуйукунуоупуруууфухуцучушущуэуюуяфгфмфнфсфуфчфюфяцмцнцоцпчвчмчсшвшршцшющрщьэгэйэмэпэфэхэшэяюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюрюфюхюцючюшюююяябядяеяжяияйякяляпяряуяцячяшяюяя',
    20 => 'ааажаиаоапафачащаюгбгвгггдгегзгкглгмгнгогсгтгугчгшгюеаежезеуефецечещеэеюииипиуихищиэиюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщлыльмбмвмгмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмымьмэмюмянанбнвнгнднжнзнкнлнннпнрнснтнунфнхнцнчншнщнъньнэнюняоаоеожошощоюоярбрврдржрзркрлрмрнрпрррсртрфрхрцрчршрщрырьрэрюрясасбсвсгсдсжсзсислсмснспсссусфсхсцсчсшсщсъсысьсэсюсятбтвтгтдтжтзтктлтптртстттутфтхтцтчтштщтътытэтюуаубувуиуйуоууухушущуяфгфлфмфнфсфтфффчфыфьфюфячачвчкчлчмчнчочрчсчтчучшчьыбывыгыдыеыжызыиыйыкылымыныпысытыуыхыцычышыщыяьбьвьгьдьжьзьиькьмьньпьсьтьфьцьчьшьщюаюбювюгюдюеюжюиюйюкюлюнюоюпюсютюфюхюцючюшющюююяябявягядяеяжязяияйялямяпярясятяуяхяцяшящяюяя',
    21 => 'ааагафацащаэбббвбгбебжбзбкблбмбнбпбрбсбтбубхбцбчбшбщбъбьбюбявбвввгвдвжвзвквлвмвнвпврвсвтвфвхвцвчвшвщвъвьвювягбгвгггдгегзгкгмгнгсгтгугчгшгюдбдвдгдддждздкдмдпдрдсдтдудфдхдцдчдшдщдъдыдьдэдяеаежезепесеуефехецечещеэеюзбзгздзезжзззизкзлзмзпзрзсзтзцзчзшзъзызьзэзюзяиуифишиэиюкгкдкекжкзкккмкнкскткукхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлфлхлцлчлшлщльмбмвмгмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюошоюпипкпмпнпппсптпфпхпцпчпшпщпыпьпюпярбрвргрдржрзркрлрмрнрпрррсртрфрхрцрчршрщрэсбсгсдсжсзснсссфсхсцсчсшсщсъсысьсэсютбтгтдтжтзтктлтмтптттфтхтцтчтштщтътьтэтюубувуиуйуоуууфуцущуэуяцацкцмцнцоцпцуцычвчкчмчнчрчсчтчшчьшашвшкшлшмшншошпшршсшушцшчшьшюъюэвэгэдэзэйэмэпэрэсэхэшэя',
    22 => 'ааабагадажазаиакаоасауафачашащаэаявбвввгвдвжвзвквлвмвнвовпврвсвтвувфвхвцвчвшвщвъвьвювяеаечещеэижихичишищиэкгкдкекжкзкккмкнкскткхкцкчкшкюмбмвмгмзмимкмлмммнмомпмрмсмтмфмхмцмчмшмщмымьмэмюмянбнвнгндненжнзнинкнлнннонпнрнснтнфнхнцнчншнщнъньнэнюняоаоеожозооопосоуофохоцочошощоэояпапепкпмпнпппсптпупфпхпцпчпшпщпыпьпюпяуаубувудуиуйуоурутуууфухучушущуэуяыбывыдыеыжызыиылымырысытыуыхычышыщыя',
    23 => 'ааазапауафацаэвбвввгвдвжвзвивквлвмвнвпврвсвтвувфвхвцвчвшвщвъвывьвювяеаежефеэиаидиииуиэквкгкдкжкзккклкмкнкркскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлулфлхлцлчлшлщлыльлюлямбмвмгмзмимкмлмммнмпмрмсмтмумфмхмцмчмшмщмымьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюоаобогодоеожозоиолооосотофоцошощоэоярарбрвргрдржрзриркрлрмрнрорпрррсртрурфрхрцрчршрщрырьрэрюрясасбсвсгсдсесжсзсислсмснспсрссстсусфсхсцсчсшсщсъсысьсэсюсятбтвтгтдтжтзтктлтнтптртстттфтхтцтчтштщтътытьтэтюузуиуоуууфуцуэшвшкшлшмшншошпшршсштшушцшчшьшюьбьвьгьдьжьзькьмьньоьпьфьцьчьшьщ',
    24 => 'ааазаоауацаэвбвввгвдвжвзвивквлвмвнвпврвсвтвфвхвцвчвшвщвъвьвювяежеэигидижиоиуичиэиюкгкдкжкзккклкмкнкркскткхкцкчкшкюлблвлглдлжлзлклллмлнлплслтлулфлхлцлчлшлщльмбмвмгмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмьмэмюнбнвнгнднжнзнкнлнннпнрнснтнфнхнцнчншнщнъньнэнюоаободожозоиолооопохоцощоэояпкпмпнпппсптпфпхпцпчпшпщпьпюпярбрвргрдржрзркрлрмрнрорпрррсртрфрхрцрчршрщрырьрэрюрясасбсвсгсдсесжсзсислсмснсоспсрссстсусфсхсцсчсшсщсъсысьсэсюсятбтвтгтдтжтзтктлтмтнтптттфтхтцтчтштщтътьтэтютяуаувудужузуиуоупуууфухуцуэцвцицкцмцнцоцпцуцычвчечичкчлчмчнчрчсчтчучшчььбьвьгьдьжьзьиькьмьньоьпьфьцьчьшьщюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюсюфюхюцючюшющюююя',
    25 => 'ааабагазаиайаоапарасауафацачашаэеаещеяиаибигидижиииоириуифиэиюиянанбнвнгнднжнзнкнлнннпнрнснтнунфнхнцнчншнщнъньнэнюняоаогодоеожозоиоколонооопоросоуофохоцочошощоэоюоярарбрвргрдржрзркрлрмрнрорпрррсртрурфрхрцрчршрщрырьрэрюуаубувугудуеужузуиуйулумуоуууфухуцушуэуяьбьвьгьдьеьжьзьиькьмьньоьпьсьтьфьцьчьшьщьюья',
    26 => 'еаебеееиейеленеоепетеуефечещеэеюеяюаюбювюдюеюжюзюиюйюкюлюмюоюпюрюфюхюцючюшющюююяябягяеяжяияйяляпяуяхяцячяшящяюяя',
    27 => 'бббвбгбдбжбзбмбпбсбтбцбшбщбъбювбвввгвдвжвзвлвмвпврвсвтвфвхвцвщвъвывьвюгбгвгггдгзгкгмгтгчгшгюдбдгдддждздкдмдпдтдфдхдцдчдщдъдьдэеаебегееежеиейекеленеоеперетеуефецечещеэеюеяжбжвжзжпжсжужфжцжчжьжюзбзззкзпзцзчзьзэзюиаибивидиеижизииийикилиминиоипиритиуифихицичишиэиюияйайбйвйгйейзйкйлймйнйойпйрйфйхйцйчйшйщйякгкдкекжкзкккмкхкцкчкюлблвлглжлзлллмлнлплтлфлхлцлшлщлюмбмвмгмммтмфмхмцмшмьмэмюнбнгнжнлнпнрнхнцншнънэпмпппсптпфпхпцпшпщпюрбржрзрмрпррртрфрхршрэрюсбсгсдсжсзсрсфсщсъсэсютбтгтдтжтзтмтптттфтхтштщтътэтюуаубувугуеузуиуйукулумунуоупурусуууфухуцушущуэуюуяхбхгхдхехзхкхмхпхрхсхцхъхэцицмцнцоцпцучвчмчрчсчшшпшршчшющнщрщьябягядяеяжязяияйякялямяняпярятяуяхяцячяшящяюяя',
    28 => 'бббвбгбдбжбзбкблбмбнбпбсбтбхбцбчбшбщбъбыбюбявбвввгвдвжвзвквлвмвнвпврвсвтвфвхвцвчвшвщвъвьвювягбгвгггдгзгкглгмгнгсгтгчгшгюдбдвдгдддждздлдндпдрдтдхдчдщдэеаебеееиелепеуехечещеэеюеяжбжвжгжджжжзжкжлжмжнжпжржсжужфжцжчжьжюзбзвзгздзжзззлзмзпзрзсзтзцзчзшзъзэзюиаибивигидиеижизииийилиоипитиуифищиэиюияквкгкдкжкзкккмкркткхкцкчкшкюмбмвмгмзмкмммнмпмрмтмфмхмцмчмшмщмымэмюнбнвнгнднжнзнкнлнннпнрнтнфнхнцнчншнщнъньнэоаобовогодоеожозоиойокомооопосоуофохоцочошощоэоюояпкпмпнпппрпспфпхпцпчпшпщпьпюпясбсвсгсдсесжсзслсмспсрсссусфсхсцсчсшсщсъсьсэсютбтвтгтдтжтзтлтхтцтчтштщтътытьтэтюфмфнфтфффчфюфяцвцкцнцпчвчкчлчмчрчтчшчьшвшкшлшмшршсштшцшчшющнщощрщущьюбювюзюйюлюоюхюцюююяябядяеязяияйяпясяуяцяшящяюяя',
    29 => 'вввгвжвзвлвнвпвсвувхвцвчвшвщвъвывьвювягбгвгггдгзгкглгмгнгргсгтгчгшгюдбдвдгдддждкдлдмдндодпдтдфдхдцдчдшдщдъдыдьдэдюдязазбзвзгздзжзззкзлзмзнзпзрзтзузчзшзъзызьзэзюзяйайбйвйгйейзймйойпйтйцйчйшйщйякгкдкжкккмкнкхкцкчкшлблвлглдлжлзлмлнлплслтлулфлхлцлчлшлщлымвмгмзмкмлмнмрмсмтмхмцмчмшмщмымьмэмюмянбнвнжнлнпнфнхншнщнънынэнюняпепкплпнппптпупфпхпцпчпшпщпыпьпярвржрлрфрхрчршрщрырьрэрюрясасбсвсгсдсжсзсислснсрсусфсцсчсшсщсъсысьсюсятбтвтгтдтжтзтмтптстттфтхтцтчтштщтътытьтэтяфгфлфмфнфофрфсфтфуфчфыфьфюфяхахбхвхгхдхехзхкхлхмхнхпхрхсхтхухцхчхшхъхэшвшишкшлшмшншошпшршсштшушцшчшьшюябявягядяеяжязяияйялямяняпярясятяуяхяцячяшящяюяя',
    30 => 'ааабавагадаеажазаиайакаламаоапасауафахацачашащаэаюаябббгбдбжбзбмбпбсбтбхбшбщбъбьбювбвввгвдвжвзвквлвмвнвпврвсвтвувфвхвцвчвшвщвъвьвювягбгвгдгзгкглгмгнгргсгтгугчгшгюдбдгдддздлдпдрдтдфдхдчдщдъдыдэдюеаебевегедееежезеиейекелеоепесеуефехецечещеэеюеяжбжвжгжджжжзжлжмжпжржужфжцжчжьжюзбзвздзззкзлзмзрзсзтзцзчзшзъзызьзэиаибивигиеижииийилиминиоиписиуифихицичишищиэиюияйайбйвйейзйлйнйойпйрйфйхйцйчйшйщйякгкдкекжклкмкркукхкцкчкшкюлблвлглдлжлзлклмлнлплтлулфлхлцлчлшлщлымвмгмзмлмрмсмтмумфмхмцмшмщмьмэмюмянбнвнжнзнлнпнрнфнчншнщнънэнюоаобовогодоеожозоиойоколомооопосотоуофохоцочошощоэоюояпеплпмпппрпсптпупфпхпцпчпшпщпыпьпюпярврдржрлрхрщрырэрюсбсвсгсдсжсзслсмспсрсусфсхсцсчсшсщсъсысэтбтвтгтдтжтзтлтмтптртттфтхтцтштщтътэтюфгфифлфмфнфофрфсфуфффчфыфьфюхбхвхгхдхехзхкхлхмхпхрхсхтхухцхчхшхъхэцацвцкцмцнцоцпцуцычвчлчмчрчсчтчшшвшлшмшпшршсштшушцшчшьшющнщрщущьюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюрюфюхюцючюшюююяябявягядяеяжязяияйякялямяняпярятяуяхяцячяшящяюяя',
    31 => 'бббвбгбдбжбзбмбпбсбтбхбцбщбъбыбювбвввгвдвжвзвмвпврвтвфвцвъвюгбгвгггзгмгюдбдгдддждздмдпдтдфдхдщдъдэеаебегееежезеиейекеленеоепересеуефехецечещеэеюеяжвжгжджжжзжлжмжпжржфжцжчжюзбздзжзззмзпзрзсзтзцзъзэзяиаибивигидиеижииийикилиоипиритиуифихишищиэиюияйайбйвйгйдйейзймйойпйрйфйхйшйщйякгкдкжкзкккмкркткхкцкчкюлблвлглдлжлзлллмлнлплулфлхлцлчлшлщмвмгмзмммпмтмфмхмчмшмэмюнбнгнжнзнлнпнрнфнхнънэплпмппптпфпхпцпшпщпюпярбрвргржрзрпррртрфрхрцрэрюсбсвсгсдсжсзсмспсрсфсхсцсчсщсъсэсютбтгтдтжтзтмтптртфтхтчтштщтътэуаубувугудуеужуиуйулумунуоупурусуууфухуцучушущуэуюуяхбхвхгхдхзхкхмхпхсхухцхшхъхэцвцмцнцоцпцуцычвчрчсчтчшшвшпшршсштшцшчшющрщьюаюбювюгюдюеюжюзюиюйюкюлюмюнюоюпюрюфюхюцючюююяябявягядяеяжязяияйякялямяняпярятяуяхяцячяшящяюяя'
    );
  $res=0;
  for($i=0;$i<strlen($s)-1;$i++)
  {
    $c1=$s[$i];
    if($c1<'а'||$c1>'я') continue;
    $c2=$s[$i+1];
    if($c2<'а'||$c2>'я') continue;
    $i1=ord($c1)-ord('а');
    if(strpos($a[$i1],$c2)!==false)
    {
      $res++;
      continue;
    }
    if($i>=strlen($s)-2) continue;
    $c3=$s[$i+2];
    if($c3<'а'||$c3>'я') continue;
    $i2=ord($c2)-ord('а');
    if(strpos($a[$i2],$c3)!==false)
    {
      $res++;
      $i++;
      continue;
    }
    $l=0;
    $r=strlen($b[$i1])/2-1;
    while($l<=$r)
    {
      $c=$l+(($r-$l)>>1);
      $ca=$b[$i1][$c*2];
      $cb=$b[$i1][$c*2+1];
      if($ca==$c2&&$cb==$c3)
      {
        $res++;
        break;
      }
      if($ca<$c2||$ca==$c2&&$cb<$c3)
        $l=$c+1;
      else
        $r=$c-1;
    }
  }
  return $res;
}

function _a_charset_ru_prepare($s)
{
  $r=0;
  $k=0;
  for($i=0;$i<min(strlen($s),1024)&&$r<128;$i++)
  {
    $c=ord($s[$i]);
    if($c>=0x80)
    {
      $r++;
      $k=$i;
    }
  }
  return substr($s,0,$k+1);
}

function _a_charset_ru_utf8($s)
{
  $r='';
  $state=1;
  for ($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    switch($state)
    {
      case 1: //not a special symbol
        if($c<=127)
          $r.=chr(0x80);
        elseif(($c&0xe0)==0xc0)
        {
          $c1=$c&0x1f;
          $state=2;
        }
        else
          $r.=chr(0x80);
        break;
      case 2: //an utf-8 encoded symbol has been meet
        $state=1;

        if(($c&0xc0)!=0x80)
        {
          $r.=chr(0x80);
          $i--;
        }
        else
        {
          $c_utf=($c1<<6)+($c&0x3f);
          if($c_utf==0x0401||$c_utf==0x0451)
            $r.='е';
          elseif($c_utf>=0x0410&&$c_utf<=0x042f)
            $r.=chr($c_utf-0x0330);
          elseif($c_utf>=0x0430&&$c_utf<=0x044f)
            $r.=chr($c_utf-0x0350);
          else
            $r.=chr(0x80);
        }
        break;
    }
  }
  return $r;
}

function a_charset_ru($s,$i_mark=1048576)
// returns:
// - an array with following fields:
//   - charset - charset name. Available character sets:
//     - windows-1251
//     - utf-8
//     - koi8-r
//     - CP866
//   - mark - mark (the greater - the worser; less then zero is the best)
// - false on failure (if i_mark is greater then specified)
{
  $s_prepared=_a_charset_ru_prepare($s);

  $a_result=array(
    'charset' => false,
    'mark' => $i_mark
    );

  $s_win1251=_a_charset_ru_cp1251($s_prepared);
  $i_mark=_a_charset_ru_good($s_win1251)*A_CHARSET_MARK_GOOD;
  if($i_mark<$a_result['mark'])
  {
    $i_mark+=_a_charset_ru_bad($s_win1251)*A_CHARSET_MARK_BAD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_ru_pair($s_win1251)*A_CHARSET_MARK_PAIR;
      if($i_mark<$a_result['mark'])
      {
        $a_result['charset']='cp1251';
        $a_result['mark']=$i_mark;
      }
    }
  }

  $s_koi8r=_a_charset_ru_koi($s_prepared);
  if(strlen($s_koi8r)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_ru_good($s_koi8r)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_ru_bad($s_koi8r)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_ru_pair($s_koi8r)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='koi8-r';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  $s_cp866=_a_charset_ru_cp866($s_prepared);
  if(strlen($s_cp866)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_ru_good($s_cp866)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_ru_bad($s_cp866)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_ru_pair($s_cp866)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='cp866';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  $s_utf8=_a_charset_ru_utf8($s_prepared);
  if(strlen($s_utf8)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_ru_good($s_utf8)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_ru_bad($s_utf8)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_ru_pair($s_utf8)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='utf-8';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  if(!$a_result['charset']) return false;
  return $a_result;
}

function _a_charset_uk_bad($s)
{ //count "bad" symbols in ukrainian, internal encoding
  $r=0;
  for($i=0;$i<strlen($s);$i++)
  {
    if(ord($s[$i])==0x80)
      $r++;
  }
  return $r;
}

function _a_charset_uk_cp1125($s)
{
  $a=array(
    //00    01    02    03    04    05    06    07    08    09    0a     0b    0c     0d    0e    0f
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0x80 - 0x8f
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff, //0x90 - 0x9f
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0xa0 - 0xaf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xb0 - 0xbf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xc0 - 0xcf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xd0 - 0xdf
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff, //0xe0 - 0xef
    0x80, 0x80, 0xe3, 0xe3, 0xfd, 0xfd, 0xfb, 0xfb, 0xfa, 0xfa, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80  //0xf0 - 0xff
  );
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=128)
      $s[$i]=chr($a[$c-128]);
    else
      $s[$i]=chr(128);
  }
  return $s;
}

function _a_charset_uk_cp1251($s)
{
  $a=array(
    //00    01    02    03    04    05    06    07    08    09    0a     0b    0c     0d    0e    0f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x80 - 0x8f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x90 - 0x9f
    0x80, 0x80, 0x80, 0x80, 0x80, 0xe3, 0x80, 0x80, 0x80, 0x80, 0xfd,  0x80, 0x80,  0x80, 0x80, 0xfa, //0xa0 - 0xaf
    0x80, 0x80, 0xfb, 0xfb, 0xe3, 0x80, 0x80, 0x80, 0x80, 0x80, 0xfd,  0x80, 0x80,  0x80, 0x80, 0xfa, //0xb0 - 0xbf
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0xc0 - 0xcf
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff, //0xd0 - 0xdf
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0xe0 - 0xef
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff  //0xf0 - 0xff
  );
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=128)
      $s[$i]=chr($a[$c-128]);
    else
      $s[$i]=chr(128);
  }
  return $s;
}

function _a_charset_uk_cp866($s)
{
  $a=array(
    //00    01    02    03    04    05    06    07    08    09    0a     0b    0c     0d    0e    0f
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0x80 - 0x8f
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff, //0x90 - 0x9f
    0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9, 0xea,  0xeb, 0xec,  0xed, 0xee, 0xef, //0xa0 - 0xaf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xb0 - 0xbf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xc0 - 0xcf
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0xd0 - 0xdf
    0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9, 0x80,  0x80, 0xfc,  0x80, 0xfe, 0xff, //0xe0 - 0xef
    0x80, 0x80, 0xfd, 0xfd, 0xfa, 0xfa, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80  //0xf0 - 0xff
  );
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=128)
      $s[$i]=chr($a[$c-128]);
    else
      $s[$i]=chr(128);
  }
  return $s;
}

function _a_charset_uk_good($s)
{ //count "good" symbols in ukrainian, internal encoding
  $r=0;
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=0xe0)
      $r++;
  }
  return $r;
}

function _a_charset_uk_koi($s)
{
  $kw = array(
    //00    01    02    03    04    05    06    07    08    09    0a     0b    0c     0d    0e    0f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x80 - 0x8f
    0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80,  0x80, 0x80,  0x80, 0x80, 0x80, //0x90 - 0x9f
    0x80, 0x80, 0x80, 0x80, 0xfd, 0x80, 0xfb, 0xfa, 0x80, 0x80, 0x80,  0x80, 0x80,  0xe3, 0x80, 0x80, //0xa0 - 0xaf
    0x80, 0x80, 0x80, 0x80, 0xfd, 0x80, 0xfb, 0xfa, 0x80, 0x80, 0x80,  0x80, 0x80,  0xe3, 0x80, 0x80, //0xb0 - 0xbf
     254,  224,  225,  246,  228,  229,  244,  227,  245,  232,  233,   234,  235,   236,  237,  238, //0xc0 - 0xcf
     239,  255,  240,  241,  242,  243,  230,  226,  252, 0x80,  231,   248, 0x80,   249,  247, 0x80, //0xd0 - 0xdf
     254,  224,  225,  246,  228,  229,  244,  227,  245,  232,  233,   234,  235,   236,  237,  238, //0xe0 - 0xef
     239,  255,  240,  241,  242,  243,  230,  226,  252, 0x80,  231,   248, 0x80,   249,  247, 0x80  //0xf0 - 0xff
     );
  for($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    if($c>=128)
      $s[$i]=chr($kw[$c-128]);
    else
      $s[$i]=chr(128);
  }
  return $s;
}

function _a_charset_uk_pair($s)
{ //count "bad" pairs of chars for a string in russian, in windows-1251
  $a=array(
    '0' => 'иь',
    '1' => 'ъь',
    '2' => 'ъ',
    '3' => 'жйхцщъья',
    '4' => 'ъ',
    '5' => 'иь',
    '6' => 'йфхшщъьэ',
    '7' => 'ъ',
    '8' => 'иь',
    '9' => 'ийь',
    '10' => 'жйъь',
    '11' => 'йшщъ',
    '12' => 'йъь',
    '13' => 'йъ',
    '14' => 'иь',
    '15' => 'бгджзйъь',
    '16' => 'ъ',
    '17' => 'жзйщъ',
    '18' => 'жйщъ',
    '19' => 'иь',
    '20' => 'вджзйхцчщъь',
    '21' => 'бжйфхцщъьэюя',
    '22' => 'джйфхчщъэ',
    '23' => 'гдзйпрфшщъьэ',
    '24' => 'джзйфчщъьэ',
    '25' => 'бвгджзйклмпрстфхцчшщъьэю',
    '26' => 'абгеиопуфъыьэюя',
    '27' => 'ь',
    '28' => 'ажилуыь',
    '29' => 'аеищыья',
    '30' => 'иуыья',
    '31' => 'аеиоыь'
    );
  $b=array(
    0 => 'ааабаеажайаоауафацачашащаъаыаэаюаябйбпбфбщбэбявфвьвэвягггзгкгпгфгчгюдйеаебевегееежезейекенеоепеуефехецечешещеъеыеэеюеяжбжвжгжжжзжпжржцжюзжзйзпзсзтзфзхзчзшзщкбкзкпкфкщкюлзмдмхмцмщмэмюоаовогоеожомоооуофоцошоыоэоюояпвпмпщпэпюпярйсгсдтбтзтпуаужуйуоуууцущуъуыуэуюуяфпфшфяхгхдхзхпцгцзцпцсцшчбчвчжчцчючяшбшршсшюшящнщяъцъщыаыбывыгыеыжыиыйыкылыоыпытыуыфыхыцычышыщыъыыыэыюыяэбэгэдэжэйэлэоэуэфэхэъэээююаюбюеюжюзюмюоюпюфюхюцющюъягяжязяпяуяфяшящяъяя',
    1 => 'аааеащаыбббвбгбдбжбзбйбкбмбнбобпбсбтбфбхбцбчбшбщбэбюбявбвввгвдвжвзвйвквлвмвнвпврвсвтвфвхвцвчвшвщвьвэвювягбгвгггдгегзгкгмгпгсгтгфгчгшгэгюдбдвдгдддйдкдлдндпдсдтдфдхдцдчдшдщдьдэдюдяеееоепеуеыеэеюжбжвжгжджжжзжкжлжмжнжожпжржсжтжужцжчжыжюжязбзвзгздзезжзззйзкзлзмзпзрзсзтзузфзхзцзчзшзщзьзэзюзяиаибиеиоипиуифиъиыиэиюйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкгкдкзкккмкнкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклмлнлслтлхлцлчмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщньнэоыпвпкпмпнпппсптпфпхпцпчпшпщпэпюпярбрвргрзрйркрлрмрпрррсртрфрхрцрчршрщрэсбсгсдспсрсссфсхсчсшсэсютбтвтгтдтзтктлтмтнтптстттфтхтцтчтштэуауеуоууущуъуыфбфгфефифкфлфмфнфпфрфсфтфуфффшфыфэфюфяхгхдхехзхкхмхнхпхрхсхтхухчхшцацбцвцгцзцкцлцмцнцпцрцсцтцццшчбчвчжчкчлчмчнчсчтчхчцчччючяшбшвшгшошршсшхшцшшшышюшящащещнщощыщяыеыиыуыщыыэбэгэдэзэйэкэмэнэоэпэрэтэуэфэхэцэчэшэъэээююаюбюгюеюжюзюйюмюнюоюпюфюхюцючюшющюъюэююябявягядяжяйякялямяняпясятяуяфяхяцячяшящяъяэяюяя',
    2 => 'аеаоаыбббвбдбжбзбйбкбмбпбсбфбхбцбчбщбэбюбявбвввгвдвжвзвйвквмвнвпврвсвтвувфвхвцвчвшвщвьвэвювягбгггдгзгкгмгпгсгтгфгчгшгэгюдбдвдгдддздйдкдлдпдсдтдфдхдцдчдшдщдэеаевеееуефеюеяжбжвжгжжжзжлжмжожпжржсжцжчжыжюжязбзжзззйзмзпзсзфзхзцзчзшзщзьзэзюиаиеиыйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкгкдкекзкпкфкхкцкчкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэпвпкпмпппспфпхпцпшпщпэпюрбрвргрдржрзрйркрлрмрнрпррртрфрхрцрчршрщрьрэсбсгсдсрсссхсцсчсшсэтбтгтдтзтлтмтптстттфтхтцтчтштэтюубуеуоупуууфуъуыуяфафбфгфифкфлфмфнфпфсфтфффшфэфюфяхгхдхзхихкхлхмхпхсхтхчхшцацбцвцгцзцкцлцмцнцпцрцсцтцуцццшчбчвчжчлчсчхчцчччючяшбшвшгшмшршсшхшцшшшюшящнщощыщяыеыиыуыыьбьвьгьдьеьзьйькьмьньпьрьсьтьфьхьцьчьшьщьъьэьюьяэбэвэгэжэзэйэкэлэмэнэоэпэсэуэфэхэцэъэююаюбювюгюдюеюжюзюйюкюлюмюнюоюпюсютюфюхюцючюшющюъюэююябявядяжяйяпярясяуяфяцяъяэяюяя',
    3 => 'ащаыбббвбгбдбебжбзбибйбкблбмбнбобпбрбсбтбубфбхбцбчбшбщбэбюбявбвввгвдвжвзвйвквлвмвнвпврвсвтвувфвхвцвчвшвщвьвэвювягагбгвгггдгзгигкглгмгнгпгргсгтгугфгчгшгыгэгюдбдвдгдддждздйдкдлдмдндпдрдсдтдудфдхдцдчдшдщдьдэдюдяеаееежеуефечещеыеюзбзвзгздзезжзззизйзкзлзмзнзозпзрзсзтзузфзхзцзчзшзщзызьзэзюзяиаивиеижиоитиуифищиъиыиэиюиякбквкгкдкзккклкмкнкпкркскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэоеоыпапвпепкплпмпнпопппсптпупфпхпцпчпшпщпыпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэсасвсгсесискслсмснсоспсрсссусфсхсцсчсшсысэсютатбтвтгтдтзтктлтмтнтптртстттутфтхтцтчтшуеуоуууфуыуяфафбфгфефифкфмфнфпфсфтфуфффшфыфэфюфячачбчвчжчкчлчмчнчочсчтчхчцчччючяшбшвшгшкшлшмшншошршсшхшцшшшышюшяыеыжыиыхыщыыэбэвэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэшэъэююаюбювюгюдюеюжюзюкюлюмюнюоюпюрюсютюфюхюцючюшющюъюэюю',
    4 => 'аааеаыаябббвбгбдбжбзбйбкбмбнбпбсбтбфбхбцбчбшбщбэбюбявбвввгвдвжвзвйвквмвнвпврвсвтвфвхвцвчвшвщвьвюгбгггдгзгкгмгпгсгтгфгчгшгэгюдбдвдгдддждйдкдлдмдпдсдтдфдхдцдчдшдщдьдэеуеэжвжджжжзжкжлжнжпжржтжцжчжюжязгздзжзззйзлзпзрзсзтзфзхзцзчзшзщзэиаиеиоиуиъиыиэиюйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкгкдкзкккмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнлнмнннпнрнснтнфнхнцнчншнщпвпкпмпнпппсптпфпхпцпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьсгсдсрсссфсхсцсчсшсэсютбтгтдтзтктлтмтптттфтхтцтчтштэтюуфуыуяфбфгфифкфлфмфнфпфсфтфуфффшфэфюфяхгхдхзхкхнхпхсхтхухчхшцбцгцзцицкцлцнцпцрцсцтцццшчбчвчжчкчлчмчнчсчтчхчцчччючяшбшгшншршсшхшцшшшюшящащнщощущяыиыфыхыцычыщыыьвьдьеьзьньпьрьфьхьцьчьшьщьъьяэбэгэдэзэйэкэмэнэоэпэрэсэтэуэфэхэцэчэшэъэююаювюдюеюзюлюоюпютюфюцюъюэююябяпясяуяфяцяшяъяя',
    5 => 'ааажайаоауафахачашащаъаыаэаюаябвбгбдбжбзбйбмбпбфбхбшбщбэбявввхвщвьвэвювягдгзгпгсгфгчгэгюдйдцдщеаебегедееежезейенеоепеуефехецечешещеъеыеэеюеяжвжгжзжпжржцжчжюзэзяйайейжйуйъйыйюйякбкгкпкфкшкщкэкюкялблдлжлзлмлплрлслхлчмгмдмжмянжожойоооуошощоэоюояпвпфпхпшпщпэпярэсгсдсрсчсэсютзтфтцтшуаугуеужуйунуоуууфухуъуыуэуюфбфгфмфпфффшфэфюфяхгцбцгцлцццшчбчжчмчсчтчхчцчючяшбшгшршхшцшшшюшящнщяъйъкъръцъщыаыбывыеыжыиыйыкыпытыуыфыхыцычышыщыъыыыэыюыяэбэгэжэйэкэоэуэхэшэъээюаюбюгюеюжюзюкюлюмюоюпюфюхюцючюшющюъябявягяжязяйяляпяуяфяцяшящяъяэяюяя',
    6 => 'аааеажаоауафащаъаыаябббвбгбдбебжбзбйбкблбмбнбпбрбсбтбфбхбцбчбшбщбыбэбявбвввгвдвевжвзвивйвквлвмвнвовпврвсвтвфвхвцвчвшвщвьвэвювягбгвгггдгегзгигкглгмгнгпгргсгтгфгчгшгэгюдбдгдддздйдкдлдмдндодпдрдсдтдфдхдцдчдшдщдьдэдюдяеаеееоеуехещеъеыеэеюеяжажбжвжгжджежжжзжижкжлжмжнжожпжржсжтжужцжчжюзбзвзгздзезжзззизйзкзлзмзнзпзрзсзтзузфзхзцзчзшзщзызьзэзюзяиаиеиоиуифиъиыиэиюиякбквкгкдкзкккмкнкпкскткфкхкцкчкшкщкэкюкялалблвлглдлжлзлклллмлнлплрлслтлфлхлцлчльлэлюмбмвмгмдмжмзмкмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщоаоеожофоцошощоыоэояпвпепипкпмпнпппсптпупфпхпцпчпшпщпыпэпюпярбрвргрдржрзрйркрлрмрнрорпрррсртрурфрхрцрчршрщрьрэрюсасбсвсгсдсискснспсрсссусфсхсцсчсшсысэсютатбтвтгтдтзтитктлтмтнтотптртстттутфтхтцтчтштытьтэтютяубудуеузуоуууфуцущуыуяцацбцвцгцзцицкцлцмцнцоцпцрцсцтцуцццшцыцьцючбчвчжчкчлчмчнчочсчтчхчцчччючяыаыбыгыдыеыжызыиыкыпырыуыфыхыцычыщыъыыюаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрютюфюхюцючюшющюъюэююябягядяжязяйякялямяняпярясяуяфяхяцяшящяъяэяюяя',
    7 => 'аебббвбгбдбжбзбйбкбмбнбпбсбтбфбхбцбчбшбщбэбявбвввгвдвжвйвквмвнвпврвсвфвхвцвчвшвщвьвюгбгггдгзгкгмгпгсгтгфгчгшгэгюдбдгдддждйдпдсдтдфдхдцдчдшдщдэдюеаееежезеуехечещеыеюжбжвжгжджжжзжкжлжнжпжржсжтжцжчжыжюжязгздзжзззйзкзпзрзсзтзфзхзцзчзшзщзызьзэзюиаибиеиоипиуихиъиыиэиюияйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкгкдкекзкккмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмммпмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэоэояпвпкпмпппсптпфпхпцпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэсбсгсдсссфсхсцсчсшсэтбтгтдтзтктмтптстттфтхтцтчтштэуеуоуууцущуыуяфбфгфефифкфлфмфнфпфрфсфтфуфффшфыфэфюфяхгхдхехзхкхнхпхсхтхухчхшцацбцгцзцицкцлцмцнцпцрцсцтцццшцьцючбчвчжчкчмчнчсчтчхчцчччючяшбшгшршсшхшцшшшышюшящнщощяыеыиыцьвьгьдьеьзьйьньпьрьфьхьцьчьшьщьъьюьяэбэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэшэъэююаюеюжюзюлюнюоюпютюфюхюцюшющюъююягядяжязяляпяряуяфяшящ',
    8 => 'ааабагадаеажайамаоапасауафахачашащаъаыаэаюаябббвбдбжбзбйбмбпбсбтбфбхбцбщбэбявввгвжвйвмвпвфвхвщвьвэвювягбгггпгчгэгюдгдйдпдтдфдхдчдщдэеаебевегедееежезейекеменеоересетеуефехецечешещеъеыеэеюеяжгжжжзжпжржтжцжюжязгзжзйзфзхзцзчзщзэзюйбйвйгйжйзйлйпйрйфйхйцйчйщйъйэйюйякбкгкзкккпкфкчкщкялвлглдлжлзлмлплрлтлфмгмжмзмтмшмщмэмюмянжнлнмнпнроаовогоеожойомонооопотоуофоцочошощоъоыоэоюояпвпппфпщпэпюпярбржрйрфсчтгтдтзтптфтштэуаубувуеужузуйулунуоупусутуууфухуцушущуъуыуэуюуяфбфгфкфпфсфффшфэфюфяхгхзхчцбцгцзцлцпцрцсцццшчбчжчсчцчюшбшршхшюшяъкънъръцъчъшъщыаыбывыгыдыеыжызыиыйыкылымыоырытыуыфыхыцычышыщыъыыыэыюыяэбэгэжэзэйэкэлэнэоэпэрэсэуэфэхэцэчэээююаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрюфюхюцюшющюъюэююябягяжяйялямяпяуяфяхяцяшящяъяя',
    9 => 'ааагадаеажайаланаоасатауафахацачашащаъаыаэаюаябббвбгбдбжбзбйбкбмбнбпбсбтбфбхбцбчбшбщбэбюбявбвгвжвзвйвквлвмвнврвсвтвфвхвцвчвшвщвьвэвювягбгвгггдгзгкгмгпгсгтгфгчгшгэгюдгдддйдкдлдмдпдсдтдфдхдцдчдшдщдьдэдюеаебевегедееежезейенеоепетеуехецечешещеъеыеэеюеяжбжгжджжжзжкжлжмжнжпжржсжтжужцжчжыжюжязбзгзжзззйзкзмзозпзсзтзузфзхзцзчзшзщзьзэзюзякгкдкзккклкпкткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмммнмпмрмтмфмхмцмчмшмщмэмюмянднжнзнкнлнмнннпнрнтнфнхнцнчнщньнэнюоаоеоооуофоцощоыоэояпвпкпмпнпппсптпхпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюрясгсдсрсссфсхсцсчсшсэсютгтдтзтктлтптттфтхтцтчтштэуаугуеузуйукунуоутуууфухуцучушущуъуыуэуюуяфбфгфкфлфмфнфпфрфсфтфффшфэфюфяхдхзхкхлхнхпхтхчхшхыцбцвцгцзцкцлцмцнцоцпцрцсцтцццшчбчвчжчкчлчнчсчтчхчцчччючяшбшгшмшншпшршсштшхшцшшшышюшящнщощяъвъдъжъзъйъкълъмъръсътъхъцъчъшъщыаыбывыгыеыжызыиыйыкылыоыпытыуыфыхыцычышыщыъыыыэыюыяэбэвэгэдэжэзэйэкэлэмэоэпэсэтэуэфэцэчэшэъээюаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрюсютюфюхюцючюшющюъюэююябявягядяжязяйяляняпятяуяфяхяцячяшящяъяэяюяя',
    10 => 'аыбабббвбгбдбебжбзбйбкблбмбнбпбсбтбубфбхбцбчбшбщбыбэбюбявбвввгвдвжвзвйвквлвмвнвпврвсвтвувфвхвцвчвшвщвьвэвюгбгвгггдгегзгигкглгмгнгпгсгтгугфгчгшгыгэгюдадбдвдгдддждздидйдкдлдмдндпдрдсдтдудфдхдцдчдшдщдыдьдэдюдяееежеоеуехечещеыеэеюеязбзвзгздзжзззйзкзлзмзнзпзрзсзтзузфзхзцзчзшзщзызьзэзюзяиаиеижиуищиыиюкбквкгкдкзккклкмкнкпкркткукфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщньнэнюпапвпепкплпмпнпппсптпупфпхпцпчпшпщпыпэпюпярвргрдрзрйрлрмрпртрхрцрчршрщрьрэсрсссхсчсштбтвтгтдтзтктлтмтптстттфтхтцтчтштэуеуъуыфбфгфефифкфлфмфнфпфрфсфтфффшфыфэфюфяхвхгхдхехзхихкхлхнхпхрхсхтхчхшхыцацбцвцгцзцкцлцмцнцоцпцрцсцтцуцццшцьцюцячбчвчжчичкчлчмчнчочсчтчхчцчччючяшбшвшгшкшлшмшншпшршсшхшцшшшюшящащещищнщущыщяыеыжыуыщыыэбэвэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэшэъэююаюгюдюеюжюзюйюкюмюнюоюпютюфюхюцючюшющюъюэююябягядяжязяйякялямяпярясятяуяфяцячяшящяъяэяюяя',
    11 => 'аеаыбббвбгбдбжбзбибйбкблбмбнбпбрбсбтбфбхбцбчбшбщбыбэбюбявбвввгвдвевжвзвивйвквлвмвнвовпврвсвтвувфвхвцвчвшвщвьвэвювягбгвгггдгзгигкглгмгпгсгтгфгчгшгэгюдбдгдддждздйдкдлдмдндпдрдсдтдудфдхдцдчдшдщдьдэдюдяеееыжбжвжгжджжжзжкжлжмжнжожпжржсжтжужцжчжыжюзазбзвзгздзезжзззизйзкзлзмзнзозпзрзсзтзфзхзцзчзшзщзызэзюзяиаиеиоириуифиъиыиэиюиякбквкгкдкзккклкмкнкпкркткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлтлулфлхлцлчмбмвмгмдмжмзмкмлмммнмпмрмтмфмхмцмчмшмщмымэмюмянанбнвнгндненжнзнкнлнмнннпнрнснтнунфнхнцнчншнщньнэнюняпвпепипкплпмпнпппсптпупфпхпцпчпшпщпэпюпярбрвргрдржрзрирйркрлрмрнрорпрррсртрурфрхрцрчршрщрырьрэрюрясасвсгсдсесискслсмснсоспсрсссфсхсцсчсшсысьсэсюсятбтвтгтдтзтктлтмтнтптртстттфтхтцтчтштьтэтютяуууыуяфбфгфифкфлфмфнфофпфрфсфтфуфффшфэфюфяхгхдхехзхихкхмхнхпхрхсхтхухчхшцацбцвцгцзцицкцлцмцнцоцпцрцсцтцуцццшцьцюцячбчвчжчкчлчмчнчочсчтчхчцчччычючяыиьюэдэжэйэкэлэоэрэуэфэхэцэчэъэюяфяя',
    12 => 'аеаыбббвбгбдбжбзбйбкбмбпбсбтбфбхбцбчбшбщбэбюбявбвввгвдвевжвзвйвквлвмвнвпвсвтвувфвхвцвчвшвщвьвэвювягбгвгггдгегзгигкглгмгпгргсгтгфгчгшгыгэгюдадбдвдгдддждздидйдкдлдмдндодпдрдсдтдудфдхдцдчдшдщдыдьдэдюдяевееепеуеыеюжбжвжгжджежжжзжкжлжмжнжожпжржсжтжужцжчжыжюжязбзвзгздзезжзззйзкзлзмзнзпзрзсзтзфзхзцзчзшзщзьзэзюзяиаиеиоиуифиъиыкбквкгкдкзккклкмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлулфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмумфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщньнэпвпмпнпппхпцпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюсбсвсгсдскслсмспсрсссфсхсцсчсшсэсютбтвтгтдтзтктмтнтптстттутфтхтцтчтштьтэтюубуоупуууъуыуяфбфгфифкфмфнфпфсфтфффшфэфюфяхвхгхдхехзхихкхмхнхохпхрхсхтхухчхшхыцацбцвцгцзцицкцлцмцнцпцрцсцтцуцццшцючбчвчжчкчлчмчнчочсчтчхчцчччючяшбшвшгшкшмшншошпшршсштшхшцшшшюшящнщощущыщяыпыыэбэвэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэъэююаюбювюгюдюеюжюйюкюмюпюсютюфюхюцючюшющюъюэююябявягядяжязяйякялямяпяряуяфяхяцячяшящяъяэяюяя',
    13 => 'аыбббвбгбдбжбзбибйбкбмбнбпбсбтбфбхбцбчбшбщбыбэбюбявбвввгвдвжвзвивйвквлвмвнвпврвсвтвфвхвцвчвшвщвьвэвювягбгггдгзгмгнгпгчгэгюдддйдфдчдщдьдэжбжвжгжджжжзжкжлжмжнжпжржсжтжцжчжюжязбзвзгздзжзззйзкзлзмзнзпзрзсзтзфзхзцзчзшзщзэзюзяиаибиеиоиуиыиякбкгкдкзкккпкхкчкшкщкэкюкялблвлглдлелжлзлклллмлнлплрлслтлулфлхлцлчльлэлюмбмвмгмдмемжмзмимкмлмммнмпмрмсмтмумфмхмцмчмшмщмымэмюмянбнвнгнднжнзнкнлнмнннпнрнтнфнхнчншнщпвпепипкпмпнпппсптпупфпхпцпчпшпщпэпюпярбрвргрдрержрзрирйркрлрмрнрпрррсртрурфрхрцрчршрщрьрэрюрясрсхсчсшсэсюсятбтдтзтмтптттфтхтцтчтштэуоуууцуыуяфбфгфифкфмфнфпфсфтфффшфэфюфяхгхдхзхихкхлхмхпхсхтхчхшцбцгцнцпцрцсцццшчбчвчкчлчмчнчсчтчхчцчччючяшбшгшкшмшпшршсшхшцшшшюшящещнщощущыщяыеыиыыьвьдьеьзьньпьрьфьхьщэбэдэжэйэкэлэпэрэтэуэфэхэцэчэшэъэююбюдюеюжюзюмюоюфюцюъяпяуяфящяъяя',
    14 => 'ааабажаоацачашащаъаыаэаюаябэбявфвьвюгбгггзгпгфгшгэгюдйдэеаебегееежейеоеуехечешещеъеыеэеюеяжгжзжпжржсжюзэзюйайгйейжйзйпйуйфйхйщйъйыйюкбкдкхкщкэкюкялрлфмвмдмхмэоаоеоооуошощоъоыоэоюояпвпмпщпэпюпярйсдсштбтгтзтхуаубувужуйуоуууфуцушуъуыуэуюуяфбфффяхпхчцгцкцнцпцшчбчжчсчючяшбшгшсшхшцшшшюшящнърыаыбывыеыжыиыйыпытыуыфыхыцычышыщыъыыыэыюыяэйэлэоэсэуэфэхэшэээююаюбюгюеюжюкюлюмюнюоюпютюфюхюцючюшющюъябяйяпяуяфяъяэяюяя',
    15 => 'аоаывавбвввгвдвевжвзвйвквлвмвнвовпврвсвтвувфвхвцвчвшвщвывьвэвювяебеееуефеыиаибиеизиоиуифиъиыиэиюкбквкгкдкзккклкмкнкпкркскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмемжмзмимкмлмммнмпмрмсмтмумфмхмцмчмшмщмымэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэпвпипкпмпнпппсптпупфпхпцпчпшпщпэпюпярбрвргрдрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэсбсвсгсдслсмснсрсссфсхсцсчсшсэтбтвтгтдтзтктлтмтптттфтхтцтчтштьтэумуоуууъуыуяфбфгфифкфлфмфнфофпфсфтфуфффшфэфюфяхвхгхдхзхкхлхмхохпхрхсхтхчхшцацбцвцгцзцкцлцмцнцоцпцрцсцтцуцццшчбчвчжчкчлчмчнчочсчтчхчцчччючяшбшвшгшкшлшмшншпшршсштшхшцшшшюшящащещнщощущыщяыбыеыиыыэбэвэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэъэээююаюбювюгюдюеюжюзюйюкюлюмюнюоюпюсютюфюхюцючюшющюъюэююябявягядяжязяйякялямяпярясяуяфяхяцячяшящяъяэяюяя',
    16 => 'бббвбгбдбжбзбйбмбпбтбфбхбчбшбщбэбявбвввгвдвжвзвйвквмвпврвтвфвхвшвщвьвэвюгбгггдгзгмгчгюдбдгдддйдмдпдтдфдхдцдчдшдщдэжвжжжцжюжязбзгздзжзззйзмзпзрзфзхзцзчзщзэзюйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкдкзкпкфкхкцкчкщкэкюкялблвлглдлжлзлклллмлнлплрлтлфлхлцлчлэмвмгмдмжмзмкмммрмфмхмчмшмщмэмюнвнднжнзнкнлнмнннрнтнфнхнчнщпвпмпппспфпхпцпчпщпэпюпярбрвргрдржрзрйркрлрмрнрпррртрурфрхрцрчршрщрьрэрюрясбсвсгсдслсмсссхсцсчсшсэсютдтхтцтчтэуоуууыфбфгфкфмфпфрфтфффшфэфюхвхгхдхзхкхпхсхчцбцвцзцкцлцмцнцпцрцсцтцуцццшчбчвчжчлчсчтчхчцчччючяшбшвшгшсшхшцшшшюшящнщяыиырыыьбьвьгьдьеьзьйьмьньпьрьсьтьфьхьцьчьшьщьъьэьюьяэбэгэдэжэйэкэмэнэоэрэсэтэуэфэчэшэъэююаюбюдюеюоюпюцюъяряуяфяъяя',
    17 => 'аоащаыбббвбгбдбжбзбибйбкблбмбнбпбсбтбфбхбцбчбшбщбэбюбявбвввгвдвжвзвйвквлвмвнвпврвсвтвувфвхвцвчвшвщвьвюгбгвгггдгзгигкглгмгнгпгсгтгфгчгшгэгюдбдвдгдддедждздйдкдлдмдндодпдрдсдтдфдхдцдчдшдщдыдьдэдюдяееежехещеыиеижиоиуиъиыиэиюиякбкгкдкзкккмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнмнннпнрнснтнфнхнцнчншнщньоаощоыпсптпфпцпчпшпщпэпюрбрвргрдрержрзрирйркрлрмрнрорпрррсртрурфрхрцрчршрщрьрэрюрясбсвсгсдскслсмснспсрссстсфсхсцсчсшсэтхтштэужуууъуыуяфбфгфифкфлфмфнфпфрфсфтфффшфэфюфяхгхдхзхкхнхпхсхтхчхшцацбцвцгцзцкцлцмцнцоцпцрцсцтцццшцючбчвчечжчкчлчмчнчочсчтчучхчцчччычючяшашбшвшгшешкшлшмшншошршсшушхшцшшшышюшяыеыжыиыуыфыцыщыыьбьвьгьдьеьзьпьрьфьхьцьшьщьъьюэбэгэжэзэйэкэлэмэнэоэпэсэтэуэфэхэцэшэъэююаюбювюеюйюлюмюоюпюфюхюцючюшющюэююябязяляпясяуяфяшящяъ',
    18 => 'аааоащаыбабббвбгбдбжбзбибйбкблбмбнбпбсбтбфбхбцбчбшбщбэбюбявбвввгвдвжвзвйвквмвнвпврвсвтвфвхвцвчвшвщвьвюгбгвгггдгзгигкглгмгнгпгсгтгфгчгшгыгэгюдбдвдгдддждздйдкдлдмдндпдрдсдтдудфдхдцдчдшдщдыдьдэдюдяеызазбзвзгздзжзззизйзкзлзмзозпзрзсзтзузфзхзцзчзшзщзызьзэзюзяиъкбквкгкдкзкккмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмумфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэпвпепипкпмпнпппсптпфпхпцпчпшпщпыпэпюпярвржрзрйрлрнртрхрцрчршрщсбсгснспсрсссусфсхсцсчсшсэсюсятбтвтгтдтзтктлтмтнтптртстттфтхтцтчтшуеуууыфбфгфифкфлфмфнфпфсфтфффшфэфюфяхвхгхдхехзхихкхмхпхрхсхтхухчхшцацбцвцгцзцкцлцмцнцоцпцрцсцтцуцццшцючбчвчжчкчлчмчнчочсчтчхчцчччючяшбшвшгшешлшмшншошпшршсшушхшцшшшышюшяыдыеызыиыфыцыщыыьвьгьдьзьйьпьрьцьчьшьщьъьюэбэгэжэйэкэмэнэоэуэфэцэчэшэъэююеюзюоюсюцюъядясяуяфяшяъяэяя',
    19 => 'ааабавагадаеажаоапауахачашащаъаыаэаюаябббжбзбйбхбщбюбявбвввгвдвжвзвйвлвмвпвтвфвхвцвчвщвьвэвювягбгггдгзгмгпгтгфгчгшгэгюдбдгдйдпдтдфдхдцдщеаебевегедееежейекемеоепеуефехечешещеъеыеэеюеяжзжпжржсжтжцжюжязжзйзмзпзсзфзхзчзшзщзэзюйайдйейжйзйпйрйуйфйхйцйчйшйщйъйыйэйюйякбкгкзкккпкфкхкшкщкэкюкялблвлжлзлмлнлплрлслфлхлцлчлэмдмжмзмхмшмщмэмюнбнлнмнпнрнфнщнэоаобовогоеожойоконооотоуофохоцочошощоъоыоэоюояпвпшпэпюпярьрэсбсгсдсрсфсхсчсштдтзтмтптфтхтцтштэуаубувугудуеужузуйукулунуоупурусутуууфухуцучушущуъуыуэуюуяфбфгфкфмфнфпфсфуфффшфэфюхдхзхпхсхчцвцгцзцлцмцпцрцсцццшчжчхчцчючяшгшршсшхшцшюшяъйъкъчъшъщыаыбыгыдыеыжызыиыйыкылымыныоыпырысытыуыфыхыцычышыщыъыыыэыюыяэбэгэдэжэзэйэкэлэоэпэуэфэхэцэчэъэээююаюбюгюдюеюжюзюкюлюмюнюоюпюрюфюхюцюшющюъябягядяжязяйякямяпяуяфяхяцяшящяъяя',
    20 => 'ааажаоапафачащаыаюбббвбгбдбебжбзбибйбкблбмбнбобпбрбсбтбубфбхбцбчбшбщбыбэбюбягбгвгггдгегзгигкглгмгнгогпгргсгтгугфгчгшгэгюеаегееежезепеуефецечещеыеюиаибигидиеижизийиоиписиуифихичишищиъиыиэиюиякбквкгкдкекзкиккклкмкнкпкркскткукфкхкцкчкшкщкыкэкюкялблвлглдлжлзлилклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмимкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнунфнхнцнчншнщньнэнюняожойооохоцощоэоюояпапвпепипкплпмпнпопппсптпупфпхпцпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюрясвсгсдсискслсмснсосрсссусфсхсцсчсшсысэсюсятбтвтгтдтзтктлтмтптртстттутфтхтцтчтштьтэтюуаубудуеуйуоупууучущуыуяфбфгфифкфлфмфнфпфрфсфтфуфффшфэфюфяшашбшвшгшешишлшмшншошпшршсштшушхшцшшшышюшяыеыжыиыуыщыыыюэбэвэгэдэжэзэйэкэлэмэнэоэпэрэсэтэуэфэхэцэчэшэъэююаюбювюгюдюеюжюйюкюлюмюнюоюпюсютюфюхюцючюшющюъюэююябявягядяжязяйяляпярясятяуяфяцячяшящяъяэяюяя',
    21 => 'агаеажафахашаывбвввгвдвжвзвйвквлвмвнвпврвсвтвфвхвцвчвшвщвьвэвюгбгвгггдгегзгигкглгмгнгогпгсгтгугфгчгшгыгэгюдадбдвдгдддждздйдкдлдмдндпдрдсдтдудфдхдцдчдшдщдыдьдэдюдяеаегееежезеуефецечещеыеэеюзбзвзгздзезжзззизйзкзлзмзнзозпзрзсзтзузфзхзцзчзшзщзьзэзюзяиаивигиеиоиуифичишиъиыиэиюиякбквкгкдкзккклкмкнкпкркскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюнбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщойоуошоыоэпвпипкплпмпнпопппсптпфпхпцпчпшпщпыпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрэсасбсгсдсискслсмснспсссусфсхсцсчсшсысэсюсятбтвтгтдтзтмтптртттфтхтцтчтштьтэтюуеужуоупуууфуцушущуъуыуячачбчвчечжчкчлчмчнчочсчтчхчцчччычючяшбшвшгшешкшлшмшншошпшршсштшушхшцшшшышюшяыеыжыиыуыфыщыыыю',
    22 => 'ааагаеажазаоасауафахацачашащаъаыаябббвбгбдбжбзбибйбкблбмбнбобпбрбсбтбфбхбцбчбшбщбыбэбюбявавбвввгвдвжвзвйвквлвмвнвпврвсвтвувфвхвцвчвшвщвюгагбгвгггдгзгигкглгмгнгогпгргсгтгугфгчгшгыгэгюеаееежеуечешещеыеэзбзгздзезжзззйзкзлзмзнзозпзрзсзтзузфзхзцзчзшзщзьзэзюиаиеижиоиуичищиъиыиэиюиякбквкгкдкекзкккмкнкпкркскткукфкхкцкчкшкщкэкюкялблвлглдлжлзлилклллмлнлолплрлслтлулфлхлцлчлыльлэлюлямбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмымэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщньнэняоаободожозойомооосотоуофохоцочошощоыоэоюояпапвпепипкплпмпнпопппсптпупфпхпцпчпшпщпыпэпюпярбрвргрдржрзрирйркрлрмрнрорпрррсртрурфрхрцрчршрщрырьрэрюрясасбсвсгсдсискслсмснсоспсрсссусфсхсцсчсшсысьсэсюсятбтгтдтзтктлтмтнтптртстттфтхтцтчтштьтэтютяуаубудуеуйуоусутуууфучушущуъуыуяцацбцвцгцецзцицкцлцмцнцпцрцсцтцуцццшцьцюцяшашбшвшгшишкшлшмшншошпшршсштшушхшцшшшышюшяыбыеыжызыиырытыфыхычышыщыыьбьгьдьеьзьйьмьпьрьсьтьфьхьцьчьшьщьъьэьюьяюаюдюеюзюлюмюнюоюпюсюфюхюшющюъябягядяжязяйякярясяуяфячяшящяъяя',
    23 => 'аеажаоафаыбббвбгбдбебжбзбибйбкблбмбнбпбрбсбтбубфбхбцбчбшбщбыбэбюбявбвввгвдвжвзвйвквлвмвнвовпврвсвтвувфвхвцвчвшвщвывьвэвюеаееежеуефецеъеыеюеяжажбжвжгжджжжзжижкжлжмжнжпжржсжтжцжчжыжюжяиаидиеиоиуифиыкбквкгкдкзккклкмкнкпкркскткфкхкцкчкшкщкэкюкялалблвлглдлжлзлклллмлнлолплрлслтлулфлхлцлчльлэлюлямбмвмгмдмжмзмкмлмммнмпмрмсмтмумфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнпнрнснтнфнхнцнчншнщньнэоаодоеожойооофоцочошощоыоэоясасбсвсгсдсесислсмснсоспсрссстсусфсхсцсчсшсысэсютбтвтгтдтзтктлтмтнтотптртстттутфтхтцтчтштытьтэтютяуеузуоуууъуыуяхвхгхдхехзхкхлхмхохпхрхсхтхухчхшхыцацбцвцгцецзцицкцлцмцнцоцпцрцсцтцуцццшцьцюцячбчвчечжчкчлчмчнчсчтчучхчцччыаыбыгыдыеыжызыиыоыуыфыхыцычыщыъыыюаюбювюгюдюеюжюзюйюкюлюмюнюоюпюсютюфюхюцючюшющюъюэююябявягядяжязяйякяляняпярясяуяфяхяцячяшящяъяэяюяя',
    24 => 'аааеажазаоащаъаыаябббвбгбдбебжбзбибйбкблбмбнбпбрбсбтбубфбхбцбчбшбщбыбэбюбявбвввгвдвжвзвйвквлвмвнвпврвсвтвувфвхвцвчвшвщвьвэвювягбгвгггдгегзгигкглгмгнгпгргсгтгугфгчгшгыгэгюееежеоеуехецещеъеыеяиаигиеижиоиуиыкбкгкдкзкккмкпкскткфкхкцкчкшкщкэкюкялблвлглдлжлзлклллмлнлплрлслтлфлхлцлчлэмбмвмгмдмжмзмкмлмммнмпмрмсмтмфмхмцмчмшмщмэмюмянбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщнэоаобожойошощоыоэояпвпкпмпнпппсптпфпхпцпчпшпщпэпюпярбрвргрдржрзрйркрлрмрнрпрррсртрфрхрцрчршрщрьрэрюрясасбсвсгсдсесискслсмснсоспсрссстсусфсхсцсчсшсысэсютбтгтдтзтктлтмтптстттфтхтцтчтштьтэтютяудуеужузуоуууфущуъуыуяхвхгхдхзхихкхлхмхнхохпхрхсхтхухчхшхыцацбцвцгцецзцицкцлцмцнцоцпцрцсцтцуцццшцьцюцяшашбшвшгшешишкшлшмшншошпшршсштшушхшцшшшыыбыгыдыеыжызыиыкыоырыуыфыхыцычышыщыъыыюаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрюсютюфюхюцючюшющюъюэююябявягядяжязяйякялямяняпярясятяуяфяхяцячяшящяъяэяюяя',
    25 => 'аааеажазаоапарауафацашащаъаыаяеаееежеоеуефехещеъеыеюеяиаибидиеижиоиуифихищиъиыиэиюиянанбнвнгндненжнзнкнлнмнннпнрнснтнунфнхнцнчншнщньнэнюняоаоеожойолоооуофоцочошощоъоыоэоюояуаубугудуеужузумуоуууфуцушущуъуыуяыаыгыдыеыжызыиыкыоыуыфыхыцыщыъыыыяябявягядяжязяйякяляняпярясяуяфяхяцячяшящяъяэяюяя',
    26 => 'вавбвввгвдвжвзвивйвлвмвпврвтвфвхвчвьвэвювядбдвдгдддждздйдмдпдрдтдфдхдцдчдшдщдэдюжбжвжгжжжзжкжлжнжожпжржцжчжыжюжязбзвзгзезжзззйзкзлзпзрзсзтзфзхзцзчзшзщзызэзюзяйайбйвйгйдйейжйзйкйлймйнйойпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбквкгкдкекзкиккклкмкнкпкркскткфкхкцкчкшкщкыкэкюкялблвлглдлжлзлллмлнлплрлслтлулфлхлцлчлэлюлямамбмвмгмдмемжмзмлмммпмрмтмумфмхмцмшмщмымэмюмянбнвнгнднжнзнлнмнпнрнтнфнхнцнчншнщньнэнюнярарбрвргрдржрзрирйркрлрмрнрорпррртрурфрхрцрчршрщрырьрэрюрясбсвсгсдсескслсмснспсрсссусфсхсцсчсшсэсютбтвтгтдтзтлтмтптртттутфтхтцтчтштэтюхвхгхдхехзхихкхлхмхпхрхсхтхухчхшхыцацбцвцгцецзцкцлцмцнцоцпцрцсцтцуцццшцюцячачбчвчечжчичлчмчочсчтчучхчцчччычючяшашбшвшгшешкшлшмшншошпшрштшушхшцшшшышюшящащищнщощущыщя',
    27 => 'аааеажайащаъаыаэаюаябббвбжбзбйбмбпбтбфбхбцбшбщбэбюбявйвьвэвювягбгггзгигкгпгфгчгэгюдэдюеаебевегееежейекенеоепересеуехецечешещеъеыеэеюеяжлжюжязжзйзфзхзчзщзэиаибивигидиеижизийикилиниоипириситиуифихицичишищиъиыиэиюияйайвйгйейжйзйпйрйуйхйыйэкгкдкзкпкхкщкэкялвлжлзлрлцмгмтмхмцмшмэмюмянмнрнэойощоэоюояпвпмпфпцпэпярзрйрфрщсбсвсдсрсхсчсэсютдтзтштэуаубувугуеужузуйукулуоутуууфухуцучушущуъуыуэуюуяфбфгфкфмфнфпфрфшфэфюфяхгхдхзхкхмхпхсхчхшцбцвцгцлцоцпцрцсцучвчжчсчтчхчцчючяшбшгшпшршсшхшцшюшящнщяъдъжъзъкълъръхъцъчъщыаыбывыгыдыеыжыиыйыкылыныоыпырысытыуыфыхыцычышыщыъыыыэыюыяэдэжэйэфэхээюаюбюгюдюеюжюзюлюмюнюоюпюрюфюхюцюшющюъябягяжяйяпяряуяфяшящяъяя',
    28 => 'бббвбгбдбжбзбйбкбмбнбпбсбтбфбхбцбчбшбщбэбювбвввгвдвжвзвйвквлвмвнвпврвсвтвфвхвцвчвшвщвьвэвювягбгвгггдгзгкглгмгнгпгргсгтгфгчгшгэгюдбдгдддздйдкдлдндпдсдхдчдщдьдэдюдяеаебегедееежезейекелеменеоепересеуефехецечешещеъеыеэеюеязбзвзгздзжзззйзкзлзмзнзозпзрзсзтзузфзхзцзчзшзщзьзэзюзяйайбйвйгйдйейжйзйкйлймйнйпйрйсйтйуйфйхйцйчйшйщйъйыйэйюйякбкгкдкзкмкткфкхкцкчкшкщкэкюкямбмвмдмжмзмкмммнмпмрмсмтмфмхмцмчмшмщмэмюнбнвнгнднжнзнкнлнмнннпнрнснтнфнхнцнчншнщньнэоеойпвпипкплпмпнпппрпспфпхпцпчпшпщпэпюпярбрвргрдрержрзрирйркрлрмрнрпрррсртрурфрхрцрчршрщрырьрэрюрясбсвсгсдсеслсмспсрсссфсхсцсчсшсэсютбтгтдтзтктлтхтцтчтштьтэтюфбфгфкфлфмфнфпфтфффшфэфюфяхахвхгхдхехзхихкхлхмхнхпхсхтхчхшцацвцкцлцнцпцрцсцтцццшчбчвчкчлчмчнчсчтчхчцчччючяшбшвшгшлшмшпшршсшхшцшшшышюшящнщыщяъдъжъзъйъкълъмъръсътъхъцъчъшъщэбэгэдэйэкэлэоэсэуэхэцэчэшэъэээююаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрюсюхюцючюшющюъюэююябявягядязяйяляпятяуяфяхяцяшящяъяэяюяя',
    29 => 'бабббвбгбдбебжбзбибйбкблбмбнбпбрбсбтбубфбхбцбчбшбщбыбэбюбявбвввжвзвйвчвшвщвьвэвювягагбгвгггдгзгкглгмгнгпгргсгтгугфгчгшгэгюдбдгдддждздйдкдмдпдрдсдтдфдхдцдчдшдщдьдюдяжажбжвжгжжжзжижкжлжмжнжпжржсжтжужцжчжыжюжязвзгзезжзззизйзкзлзмзпзрзсзтзфзхзцзчзшзщзызьзэзюзяйайбйвйгйдйейжйзйкйлймйнйойпйрйтйуйфйхйцйчйшйщйъйыйэйюйякбквкгкдкекзккклкмкнкпкскукфкхкчкшкщкыкэкюкялблвлдлжлзлмлнлплрлслтлулфлхлцлчмбмвмгмдмжмзмммпмрмтмфмхмчмшмщмэмюмянвнгнжнзнлнмнпнрнфнхнчншнщнэнюоаобовогодоеожозойоколомонооопосотоуофохоцошощоъоыоэоюояпвпепкплпмпппсптпупфпхпцпчпшпщпэпюпярзрйрлрпртрфрхрцрчрщрьрюрясбсвсгсдсмснсосрсссфсцсчсшсысэсюсятбтгтдтзтмтптутфтхтцтчтштытэтютяуаубугудуеужузуйулумунуоупурусутуууфухуцучушущуъуыуэуюуяфафбфгфефкфлфмфофпфсфтфуфффшфэфюфяхахвхгхдхехзхкхлхмхнхпхрхсхтхухчхшцацбцвцгцецзцицкцлцмцнцоцпцрцсцтцуцццшцыцюцячбчвчжчичлчмчочсчтчучхчцчччычючяшашбшвшгшишлшмшошпшрштшушхшцшшшышюшяъвъжъзъйъкълъмънъръсътъхъцъчъшъщэбэгэдэжэзэйэкэлэмэоэпэрэсэтэуэфэхэцэчэшэъэээююаюбювюгюдюеюжюзюйюкюлюмюнюоюпюрюсютюфюхюцючюшющюъюэюю',
    30 => 'ааабавагадаеажазайакаламаоапасауафахацачашащаъаыаэаюаябббвбгбдбжбзбйбмбпбтбфбхбшбщбэбюбявбвввгвдвжвзвивйвквлвмвнвпврвсвтвувфвхвцвчвшвщвьвэвювягбгвгггдгзгкглгмгпгсгтгфгчгшгэгюдбдгдздйдлдпдрдтдфдхдчдшдщдэдюеаебевегедееежезейекелемеоепересетеуефехецечешещеъеыеэеюеяжбжвжгжджжжзжлжмжпжржсжтжужцжчжыжюжязбзвздзжзззйзлзмзпзрзсзтзфзхзцзчзшзщзэзюзяйайбйвйгйейжйзйлйнйойпйрйуйфйхйцйчйшйщйъйыйэйюйякбквкгкдкзккклкмкпкркфкхкцкчкшкщкэкюкялблвлглдлжлзлклмлнлплрлслтлулфлхлцлчлэмвмгмдмзмкмлмммрмтмфмхмцмчмшмщмэмюмянбнвнжнзнлнмнпнрнснфнчншнщнэнюоаобовогодоеожойоколомооопосотоуофохоцочошощоъоыоэоюояпвпепмпппрпсптпфпхпцпшпщпэпюпярврдржрзрйрфрхрцршрщрьрэрюсбсвсгсдсесмспсрсфсхсцсчсшсэтбтвтгтдтзтктмтптртстттфтхтцтчтштэтяфбфгфифкфлфмфнфофпфрфсфуфффшфыфэфюфяхвхгхдхзхлхмхпхрхсхчхшцацбцвцгцзцкцлцмцнцоцпцрцсцтцуцццшцюцячбчвчжчлчмчсчтчхчцчючяшбшвшгшлшмшпшршсштшхшцшшшюшящнщяъвъжъйъкълъмънъръсъхъцъчъшъщэбэвэгэдэжэзэйэкэлэнэоэпэрэсэуэфэхэцэчэъэээююаювюгюдюеюжюзюйюкюлюмюнюоюпюрюфюхюцюшющюъюэюю',
    31 => 'бббвбгбдбжбзбйбмбпбсбтбфбхбцбшбщбэбюбявбвввгвжвзвйвмвпврвтвфвцвщвьвэвювягбгвгггдгзгкгмгпгфгчгэгюдбдгдздйдмдпдтдфдхдцдшдщдэжбжвжгжжжзжпжржцжюжязбзвзгздзжзззйзмзпзрзсзфзхзцзчзщзяйайбйвйгйдйейжйзйойрйуйфйхйшйщйъйыйэйюйякгкзкккмкпкркскфкхкцкэкюкялблвлглдлжлзлллмлнлплрлслулфлхлцлчлэмвмгмдмжмзмммтмфмхмцмчмшмщмэмюмянбнвнжнзнлнпнрнфншнэпвпепмппптпфпхпцпчпшпщпэпюпярбрвргржрзрйрпррртрфрхршрэрюсбсвсгсдсмспсрсфсхсчсэсюсятбтгтзтмтптфтхтцтчтштэтюуаубувугудуеужуйукулумунуоупурусуууфухуцучушущуъуыуэуюуяфафбфгфифкфлфмфнфофпфрфсфтфуфффшфыфэфюфяхгхдхзхихмхпхрхсхчхшцацбцвцгцзцлцмцнцоцпцрцсцуцццшчбчвчжчсчхчцчюшбшвшгшлшпшршсштшушхшцшшшюшящнщяъдъжъзъйъкълъмърътъхъчъшъщэбэгэдэжэзэйэкэлэнэоэпэрэсэуэфэхэцэъэээююаюбюгюдюеюжюзюйюкюлюмюнюоюпюрюфюхюцюшющюъябягядяжязяйякяляпярясяуяфяхяцячяшящяъяэяюяя'
    );
  $res=0;
  for($i=0;$i<strlen($s)-1;$i++)
  {
    $c1=$s[$i];
    if($c1<'а'||$c1>'я') continue;
    $c2=$s[$i+1];
    if($c2<'а'||$c2>'я') continue;
    $i1=ord($c1)-ord('а');
    if(strpos($a[$i1],$c2)!==false)
    {
      $res++;
      continue;
    }
    if($i>=strlen($s)-2) continue;
    $c3=$s[$i+2];
    if($c3<'а'||$c3>'я') continue;
    $i2=ord($c2)-ord('а');
    if(strpos($a[$i2],$c3)!==false)
    {
      $res++;
      $i++;
      continue;
    }
    $l=0;
    $r=strlen($b[$i1])/2-1;
    while($l<=$r)
    {
      $c=$l+(($r-$l)>>1);
      $ca=$b[$i1][$c*2];
      $cb=$b[$i1][$c*2+1];
      if($ca==$c2&&$cb==$c3)
      {
        $res++;
        break;
      }
      if($ca<$c2||$ca==$c2&&$cb<$c3)
        $l=$c+1;
      else
        $r=$c-1;
    }
  }
  return $res;
}

function _a_charset_uk_prepare($s)
{
  $r=0;
  $k=0;
  for($i=0;$i<min(strlen($s),1024)&&$r<128;$i++)
  {
    $c=ord($s[$i]);
    if($c>=0x80)
    {
      $r++;
      $k=$i;
    }
  }
  return substr($s,0,$k+1);
}

function _a_charset_uk_utf8($s)
{
  $r='';
  $state=1;
  for ($i=0;$i<strlen($s);$i++)
  {
    $c=ord($s[$i]);
    switch($state)
    {
      case 1: //not a special symbol
        if($c<=127)
          $r.=chr(128);
        elseif(($c&0xe0)==0xc0)
        {
          $c1=$c&0x1f;
          $state=2;
        }
        else
          $r.=chr(0x80);
        break;
      case 2: //an utf-8 encoded symbol has been meet
        $state=1;

        if(($c&0xc0)!=0x80)
        {
          $r.=chr(0x80);
          $i--;
        }
        else
        {
          $c_utf=($c1<<6)+($c&0x3f);
          if($c_utf>=0x0410&&$c_utf<=0x0429||$c_utf==0x42c||$c_utf==0x042e||$c_utf==0x042f)
            $r.=chr($c_utf-0x0330);
          elseif($c_utf>=0x0430&&$c_utf<=0x0449||$c_utf==0x44c||$c_utf==0x044e||$c_utf==0x044f)
            $r.=chr($c_utf-0x0350);
          elseif($c_utf==0x0406||$c_utf==0x0456)
            $r.='ы';
          elseif($c_utf==0x0490||$c_utf==0x0491)
            $r.='г';
          elseif($c_utf==0x0404||$c_utf==0x0454)
            $r.='э';
          elseif($c_utf==0x0407||$c_utf==0x0457)
            $r.='ъ';
          else
            $r.=chr(0x80);
        }
        break;
    }
  }
  return $r;
}

function a_charset_uk($s,$i_mark=1048576)
// returns:
// - an array with following fields:
//   - charset - charset name. Available character sets:
//     - windows-1251
//     - utf-8
//     - koi8-u
//     - CP866
//     - CP1125
//   - mark - mark (the greater - the worser; less then zero is the best)
// - false on failure (if i_mark is greater then specified)
{
  $s_prepared=_a_charset_uk_prepare($s);

  $a_result=array(
    'charset' => false,
    'mark' => $i_mark
    );

  $s_win1251=_a_charset_uk_cp1251($s_prepared);
  $i_mark=_a_charset_uk_good($s_win1251)*A_CHARSET_MARK_GOOD;
  if($i_mark<$a_result['mark'])
  {
    $i_mark+=_a_charset_uk_bad($s_win1251)*A_CHARSET_MARK_BAD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_uk_pair($s_win1251)*A_CHARSET_MARK_PAIR;
      if($i_mark<$a_result['mark'])
      {
        $a_result['charset']='cp1251';
        $a_result['mark']=$i_mark;
      }
    }
  }

  $s_koi8u=_a_charset_uk_koi($s_prepared);
  if(strlen($s_koi8u)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_uk_good($s_koi8u)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_uk_bad($s_koi8u)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_uk_pair($s_koi8u)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='koi8-u';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  $s_cp866=_a_charset_uk_cp866($s_prepared);
  if(strlen($s_cp866)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_uk_good($s_cp866)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_uk_bad($s_cp866)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_uk_pair($s_cp866)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='cp866';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  $s_cp1125=_a_charset_uk_cp1125($s_prepared);
  if(strlen($s_cp1125)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_uk_good($s_cp1125)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_uk_bad($s_cp1125)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_uk_pair($s_cp1125)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='cp1125';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  $s_utf8=_a_charset_uk_utf8($s_prepared);
  if(strlen($s_utf8)*A_CHARSET_MARK_GOOD<$a_result['mark'])
  {
    $i_mark=_a_charset_uk_good($s_utf8)*A_CHARSET_MARK_GOOD;
    if($i_mark<$a_result['mark'])
    {
      $i_mark+=_a_charset_uk_bad($s_utf8)*A_CHARSET_MARK_BAD;
      if($i_mark<$a_result['mark'])
      {
        $i_mark+=_a_charset_uk_pair($s_utf8)*A_CHARSET_MARK_PAIR;
        if($i_mark<$a_result['mark'])
        {
          $a_result['charset']='utf-8';
          $a_result['mark']=$i_mark;
        }
      }
    }
  }

  if(!$a_result['charset']) return false;
  return $a_result;
}

function a_charset($s)
// returns:
// - an array with following fields:
//   charset - charset name. Available character sets:
//     - utf-8
//     - koi8-r
//     - koi8-u
//     - cp866
//     - cp1125
//     - cp1251
//   language - language code. Available languages:
//     - uk Ukraine
//     - ru Russian
//   mark - mark (the greater - the worser; less then zero is the best)
// - false on failure
{
  $a_result=array(
    'charset' => false,
    'mark' => 1048576,
    'language' => false
    );

  $a=a_charset_ru($s,$a_result['mark']);
  if($a)
  {
    $a_result['charset']=$a['charset'];
    $a_result['mark']=$a['mark'];
    $a_result['language']='ru';
  }

  $a=a_charset_uk($s,$a_result['mark']);
  if($a)
  {
    $a_result['charset']=$a['charset'];
    $a_result['mark']=$a['mark'];
    $a_result['language']='uk';
  }
  if(!$a_result['charset']) return false;
  return $a_result;
}

function str2utf($s)
{
  $a = a_charset($s);
  return @iconv($a['charset'], 'utf-8//IGNORE', $s);
}

function str2win($s)
{
  $a = a_charset($s);
  return @iconv($a['charset'], 'windows-1251//IGNORE', $s);
}

?>